<?php

namespace App\Controllers\Api;

use LMVC\System\Request;
use LMVC\System\Response;
use \DateTime;

libxml_use_internal_errors(true);

class CustomPortalTicket extends \LMVC\System\Controller
{
    protected $customPortalModel;
    protected $logModel;
    protected $uploadFolder = '';

    public function __construct()
    {
        parent::__construct();

        $serverPort= getCgiPort();
        $this->LogModel = $this->model('LogModel');

        $this->qcloudModel = $this->model('QCloudModel');
        $this->customPortalModel = $this->model('CustomPortalModel');
        $this->fileLogModel = $this->model('FileLogModel');
        $this->qtsModel = $this->model('QTSModel');
        $this->qtsModel->qtsApiPort = $serverPort;
        $this->customPortalModel->qtsApiPort = $serverPort;
        $this->customPortalModel->qcloudModel = $this->qcloudModel;
        $this->customPortalModel->fileLogModel = $this->fileLogModel;
        $this->remoteAccessModel = $this->model('RemoteAccessModel');

        if(!$this->qtsModel->isCSRFVerify()){
            $this->fileLogModel->logError('CSRF error', __FILE__);
            Response::error(403, apiFormater("Fail", 2020));
        }

        if (!$this->qtsModel->isValidUser()) {
            $this->fileLogModel->logError('You do not have permission to use this API.', __FILE__);
            Response::error(403, apiFormater("Fail", 2000));
        }

        $this->customPortalModel->qtsApiPort = $serverPort;

        $this->uploadFolder = config('app_dir') . '/data/tmp';
    }
    public function getSystemLog()
    {
        $file = $this->customPortalModel->generateSystemLog();

        if (file_exists($file)) {
            $info=pathinfo($file);
            Response::json(array('downloadPath' => '/api/v1/log/download/'.$info['filename']));
        } else {
            Response::error(500, 'Cannot generate system log. ');
        }
    }
    /* }}} */

    public function downloadSystemLog($file = '')
    {
        $name = $file;
        $file = "/tmp/helpdump/{$file}.zip";
        if (file_exists($file)&&!is_dir($file)) {
            header('Content-Description: File Transfer');
            header('Content-Type: application/force-download');
            header('Content-Type: application/octet-stream');
            header('Content-Disposition: attachment; filename="' . basename($file) . '"');
            header('Expires: 0');
            header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
            header('Pragma: public');
            header('Content-Length: ' . filesize($file));
            header('Content-Transfer-Encoding: binary ');
            readfile($file);
            if (file_exists($file)) {
                shell_exec("qsh helpdeskd.del_log  -v '{\"name\":\"".$name."\"}'");
            }
        } else {
            Response::error(500, 'Cannot download system log. ');
        }
    }
    public function getFirmwareVersion($lang = '')
    {
        $this->customPortalModel->lang = $lang;
        $arrIssueCategory = $this->customPortalModel->getFirmwareVersion();
       
        if (!is_array($arrIssueCategory) && $arrIssueCategory == false) {
            Response::error(500, apiFormater([], 2001));
        } else {
            $objIssueCategory = json_encode($arrIssueCategory);
            $objIssueCategory = json_decode($objIssueCategory);

            if (isset($objIssueCategory->status)) {
                Response::error($objIssueCategory->status, apiFormater($objIssueCategorys->msg, 2014));
            }
        }


        Response::json($arrIssueCategory);
    }

    public function getCountry($lang = '')
    {
        $this->customPortalModel->lang = $lang;
        $arrIssueCategory = $this->customPortalModel->getCountry();
       
        if (!is_array($arrIssueCategory) && $arrIssueCategory == false) {
            Response::error(500, apiFormater([], 2001));
        } else {
            $objIssueCategory = json_encode($arrIssueCategory);
            $objIssueCategory = json_decode($objIssueCategory);

            if (isset($objIssueCategory->status)) {
                Response::error($objIssueCategory->status, apiFormater($objIssueCategorys->msg, 2014));
            }
        }


        Response::json($arrIssueCategory);
    }

    public function getCustomer($lang = '')
    {
        $this->customPortalModel->lang = $lang;
        $arrIssueCategory = $this->customPortalModel->getCustomer();
       
        if (!is_array($arrIssueCategory) && $arrIssueCategory == false) {
            Response::error(500, apiFormater([], 2001));
        } else {
            $objIssueCategory = json_encode($arrIssueCategory);
            $objIssueCategory = json_decode($objIssueCategory);

            if (isset($objIssueCategory->status)) {
                Response::error($objIssueCategory->status, apiFormater($objIssueCategorys->msg, 2014));
            }
        }


        Response::json($arrIssueCategory);
    }

    public function getIssueType($lang = '')
    {
        $this->customPortalModel->lang = $lang;
        $arrIssueCategory = $this->customPortalModel->getIssueType();
       
        if (!is_array($arrIssueCategory) && $arrIssueCategory == false) {
            Response::error(500, apiFormater([], 2001));
        } else {
            $objIssueCategory = json_encode($arrIssueCategory);
            $objIssueCategory = json_decode($objIssueCategory);

            if (isset($objIssueCategory->status)) {
                Response::error($objIssueCategory->status, apiFormater($objIssueCategorys->msg, 2014));
            }
        }

        Response::json($arrIssueCategory);
    }
    public function getIssueCategory($lang = '')
    {
        $this->customPortalModel->lang = $lang;
        $arrIssueCategory = $this->customPortalModel->getIssueCategory();
       
        if (!is_array($arrIssueCategory) && $arrIssueCategory == false) {
            Response::error(500, apiFormater([], 2001));
        } else {
            $objIssueCategory = json_encode($arrIssueCategory);
            $objIssueCategory = json_decode($objIssueCategory);

            if (isset($objIssueCategory->status)) {
                Response::error($objIssueCategory->status, apiFormater($objIssueCategorys->msg, 2014));
            }
        }

        Response::json($arrIssueCategory);
    }

    public function getOs($lang = '')
    {
        $this->customPortalModel->lang = $lang;
        $arrIssueCategory = $this->customPortalModel->getOs();
       
        if (!is_array($arrIssueCategory) && $arrIssueCategory == false) {
            Response::error(500, apiFormater([], 2001));
        } else {
            $objIssueCategory = json_encode($arrIssueCategory);
            $objIssueCategory = json_decode($objIssueCategory);

            if (isset($objIssueCategory->status)) {
                Response::error($objIssueCategory->status, apiFormater($objIssueCategorys->msg, 2014));
            }
        }


        Response::json($arrIssueCategory);
    }

    public function getTimezone($lang = '')
    {
        $this->customPortalModel->lang = $lang;
        $arrIssueCategory = $this->customPortalModel->getTimezone();
        if (!is_array($arrIssueCategory) && $arrIssueCategory == false) {
            Response::error(500, apiFormater([], 2001));
        } else {
            $objIssueCategory = json_encode($arrIssueCategory);
            $objIssueCategory = json_decode($objIssueCategory);

            if (isset($objIssueCategory->status)) {
                Response::error($objIssueCategory->status, apiFormater($objIssueCategorys->msg, 2014));
            }
        }
        foreach($arrIssueCategory as $k=>$v){
            $arrIssueCategory[$k]->API_KEY = $v->sf_key;
            $arrIssueCategory[$k]->VALUE = $v->sf_key;
        }


        Response::json($arrIssueCategory);
    }

    public function getSystemInfo()
    {
        $sysInfo = $this->customPortalModel->getSystemInfo();
        $hwModel = $sysInfo['hwModel'];
        $hwVer = $sysInfo['hwVer'];
        $hwSN = $sysInfo['hwSN'];
        Response::json(array(
            'modelName' => $hwModel,
            'firmwareVer' => $hwVer,
            'serialNum' => $hwSN,
        ));
    }

   

    /* {{{ public function submitTicket() { */
    public function submitTicket()
    {
        $sysInfo = $this->customPortalModel->getSystemInfo();
        
        $qnap_user_id = trim(Request::any('qnap_user_id'));
        $qnap_phone = trim(Request::any('qnap_phone'));
        $qnap_email = trim(Request::any('qnap_email'));

        $email = trim(Request::any('email'));
        $firstName = trim(Request::any('first_name'));
        $lastName = trim(Request::any('last_name'));

        $category1 = trim(Request::any('category1'));
        $category2 = trim(Request::any('category2'));
        $category3 = trim(Request::any('category3'));
        $osType = trim(Request::any('osType'));

        $country= trim(Request::any('country'));
        $customer= trim(Request::any('customer'));
        $phone=  trim(Request::any('phone'));
        $company= trim(Request::any('company'));

        $subject = trim(Request::any('subject'));
        $contents = trim(Request::any('contents'));
        $attachSysLog = trim(Request::any('attachlog'));

        $timezone = Request::any('timezone');
        $timezone = json_encode($timezone);
        $timezone = json_decode($timezone);

        $isUpdateQid =  trim(Request::any('updateQid'));

        $model = $sysInfo['hwModel'];
        $hwSN = $sysInfo['hwSN'];
        
        $this->customPortalModel->FirmwareVersion = $sysInfo['hwVer'];
        $hwVer = $this->customPortalModel->getFirmwareVersionID();
        //$hddModel = null;
        //$raidCfg = $storageInfo['poolInfo'];
        
        $attachedFile = array();
        $attachedFileName = array();
        $arrUploadFailedFile = array();

        $postID = '';
        $logPath = '';
        
        
        // Create ticket and get ticket ID

        $this->customPortalModel->qnap_user_id = $qnap_user_id;
        $this->customPortalModel->qnap_email = $qnap_email;
        $this->customPortalModel->qnap_phone = $qnap_phone;

        $this->customPortalModel->issueCategory1 = $category1;
        $this->customPortalModel->issueCategory2 = $category2;
        $this->customPortalModel->issueCategory3 = $category3;
        $this->customPortalModel->country= $country;
        $this->customPortalModel->customer= $customer;
        $this->customPortalModel->phone=  $phone;
        $this->customPortalModel->company= $company;
        $this->customPortalModel->osType=$osType;

        $this->customPortalModel->email = $email;
        
        $this->customPortalModel->firstName = $firstName;
        $this->customPortalModel->lastName = $lastName;

        $this->customPortalModel->timezone = $timezone;
        
        $this->customPortalModel->caseSubject = $subject;
        $this->customPortalModel->description = $contents;

        if ($hwVer === false) {
            $this->customPortalModel->hwVer = 'Others';
        } else {
            $this->customPortalModel->hwVer = $hwVer;
        }
        $this->customPortalModel->model = $model;
        $this->customPortalModel->hwSN = $hwSN;



        $customPortalTicketInfo = $this->customPortalModel->createTicket();
        
        if (empty($customPortalTicketInfo['result'])||$customPortalTicketInfo['result']!="Success") {
            $this->LogModel->qtsLog(1);
            $this->LogModel->setAppLog('notification_submitTicket_failed');
            if(isset($customPortalTicketInfo['msg'])){
                Response::error(403, apiFormater($customPortalTicketInfo['msg'],3003));
            }else{
                Response::error(400, 'SubmitTicket failed');
            }
            
            return;
        }

        $this->customPortalModel->caseId = $customPortalTicketInfo['caseId'];
        $this->customPortalModel->contactId = $customPortalTicketInfo['contactId'];
        $this->customPortalModel->replyRecordId = $customPortalTicketInfo['replyRecordId'];

        // Attach system log
        if ( (isset($customPortalTicketInfo['caseNo']) && !empty($customPortalTicketInfo['caseNo'])) && $attachSysLog ) {
            $logPath = $this->customPortalModel->generateSystemLog();
            $this->customPortalModel->file = curl_file_create($logPath);

            $uploadResult = $this->customPortalModel->uploadTicketAttachment();

            if ($uploadResult === true) {
                array_push($attachedFileName, basename($logPath));
            }
        }

        // Attach external app log
        if ((isset($customPortalTicketInfo['postID']) && !empty($customPortalTicketInfo['postID'])) && $attachSysLog) {
            $arrLogPath = $this->customPortalModel->getExternalLog();
            foreach ($arrLogPath as $log) {
                $this->customPortalModel->file = curl_file_create($log);
                $uploadResult = $this->customPortalModel->uploadTicketAttachment();
                if ($uploadResult === true) {
                    array_push($attachedFileName, basename($log));
                }
            }
        }
        // User attachment
        if (file_exists($this->uploadFolder) && is_dir($this->uploadFolder)) {
            if ($handle = opendir($this->uploadFolder)) {
                while (false !== ($entry = readdir($handle))) {
                    if ($entry != '.' && $entry != '..') {
                        array_push($attachedFile, array(
                            'fileName' => $entry,
                            'fileContent' => base64_encode(file_get_contents($this->uploadFolder . '/' . $entry)),
                            'filePath' =>  $this->uploadFolder. '/' . $entry,
                        ));
                    }
                }
                closedir($handle);
            }
        }

        if (!empty($attachedFile)) {
            foreach ($attachedFile as $file) {
                //$this->customPortalModel->fileName = $file['fileName'];
                //$this->customPortalModel->fileContent = $file['fileContent'];
                $this->customPortalModel->file = curl_file_create($file['filePath']);
                $uploadResult = $this->customPortalModel->uploadTicketAttachment();

                if ($uploadResult !== true && is_array($uploadResult)) {
                    array_push($arrUploadFailedFile, $file['fileName']);
                    $this->fileLogModel->logDebug("{$file['fileName']}: {$uploadResult['msg']}");
                } elseif ($uploadResult === false) {
                    array_push($arrUploadFailedFile, $file['fileName']);
                    $this->fileLogModel->logDebug("{$file['fileName']}: Unknown Error");
                } else {
                    array_push($attachedFileName, $file['fileName']);
                }
            }
        }

        $attachedFile = null;

        if (file_exists($logPath)) {
            shell_exec("qsh helpdeskd.del_log  -v '{\"name\":\"".$hwSN."\"}'");
        }

        if (file_exists($this->uploadFolder)) {
            shell_exec("qsh helpdeskd.del_tmpfile '{\"app_path\":\"".config('app_dir')."\"}'");
        }

        if($customPortalTicketInfo['caseNo'] 
            && $customPortalTicketInfo['caseNo']!=null 
            && $customPortalTicketInfo['caseNo'] != ""){
            $this->LogModel->qtsLog(2,$customPortalTicketInfo['caseNo'],$subject);
            $this->LogModel->setAppLog('notification_submitTicket_success', array('TOKEN_TICKET' => "#{$customPortalTicketInfo['caseNo']}", 'TOKEN_SUBJECT' => "{$subject}"));
        }
        $userinfo = false;
        if($isUpdateQid){
            $this->qcloudModel->firstName = $firstName;
            $this->qcloudModel->lastName = $lastName;
            $this->qcloudModel->country = $country;
            $this->qcloudModel->timezone = $timezone;
            $userinfo = $this->qcloudModel->setUserInfo();
        }
        

        Response::json(array(
            'ticketid' => $customPortalTicketInfo['caseNo'],
            'attachment' => $attachedFileName,
            'failedFile' => $arrUploadFailedFile,
            'updateQid' => $userinfo,
        ));
    }
    /* }}} */

    public function uploadAttachment()
    {
        $fileList = array();
        $arrAllowedFileType = array(
            'text/plain',
            'text/csv',
            'text/tsv',
            'image/jpeg',
            'image/x-ms-bmp',
            'image/tiff',
            'image/png',
            'application/msword',
            'application/vnd.ms-excel',
            'application/vnd.ms-powerpoint',
            'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
            'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
            'application/vnd.openxmlformats-officedocument.presentationml.presentation',
            'application/pdf',
            'application/vnd.oasis.opendocument.text',
            'application/zip',
            'application/x-rar'
        );
        $arrAllowedFileExt = array(
            '.txt', '.csv',
            '.jpg', '.jpeg', '.bmp', '.gif', '.png',
            '.doc', '.xls', '.ppt', '.pps',
            '.docx', '.xlsx', '.pptx', '.ppsx',
            '.pdf', '.odt',
            '.zip', '.rar'
        );

        // Check and create upload destination
        if (!file_exists($this->uploadFolder)) {
            mkdir($this->uploadFolder, 0700);
        }

        // If upload destination is not folder, remove it
        if (file_exists($this->uploadFolder) && !is_dir($this->uploadFolder)) {
            shell_exec("qsh helpdeskd.del_tmpfile '{\"app_path\":\"".config('app_dir')."\"}'");
            mkdir($this->uploadFolder, 0700);
        }

        foreach ($_FILES['attachFile']['error'] as $key => $err) {
            if ($err == UPLOAD_ERR_OK) {
                $tmpName = $_FILES['attachFile']['tmp_name'][$key];
                if (!$tmpName) {
                    continue;
                }

                $fileName = $_FILES['attachFile']['name'][$key];
                $fileExt = mb_substr($fileName, mb_strrpos($fileName, '.'));
                $fileMimeType = mime_content_type($_FILES['attachFile']['tmp_name'][$key]);

                // Check file size
                //if ($_FILES['attachFile']['size'][$key] > 5242880) {
                if ($_FILES['attachFile']['size'][$key] > 25*1024*1024) {
                    Response::error(400, apiFormater(array('fileName' => $fileName), 2015));
                }

                

                // Check file type
                // if (!in_array($fileMimeType, $arrAllowedFileType) || !in_array($fileExt, $arrAllowedFileExt)) {
                    // $this->fileLogModel->logError("Disallow file type: {$fileMimeType}, Filename: {$fileName}, FileExt: {$fileExt}");
                    // Response::error(400, apiFormater(array('fileName' => $fileName), 2016));
                // }

                if (file_exists($this->uploadFolder . '/' . $fileName)) {
                    $fileName = preg_replace("/$fileExt/", '', $fileName) . '_' . date('His', time()) . $fileExt;
                }

                if (move_uploaded_file($tmpName, $this->uploadFolder . '/' . $fileName)) {
                    array_push($fileList, $fileName);
                } else {
                    Response::error(500, apiFormater(array('fileName' => $fileName), 2017));
                }
            }
            if($err == UPLOAD_ERR_INI_SIZE){
                $fileName = $_FILES['attachFile']['name'][$key];
                Response::error(400, apiFormater(array('fileName' => $fileName), 2015));
            }
        }

        shell_exec(config('app_dir') . '/data/script/qdesk clean-temp enable 30');

        if (count($fileList) !== 1) {
            Response::json($fileList);
        } else {
            Response::json(apiFormater(array('fileName' => $fileList[0]), 0));
        }
    }
    public function removeAttachment($file = '')
    {
        $fileName = empty($file) ? trim(Request::any('filename')) : $file;

        if (empty($fileName)) {
            if (file_exists($this->uploadFolder)) {
                foreach (glob($this->uploadFolder . '/{,.}*', GLOB_BRACE) as $f) {
                    if (is_file($f)) {
                        unlink($f);
                    }
                }
                try {
                    rmdir($this->uploadFolder);
                } catch (Exception $e) {
                    // echo $e->getMessage();
                }
            }

            shell_exec(config('app_dir') . '/data/script/qdesk clean-temp disable');

            Response::json('Folder ' . $this->uploadFolder . ' removed');
        } else {
            if (file_exists($this->uploadFolder . '/' . $fileName)) {
                unlink($this->uploadFolder . '/' . $fileName);
                Response::json('File ' . $fileName . ' removed');
            } else {
                Response::error(404, $fileName . ' not found');
            }
        }
    }



}
