<?php

namespace App\Controllers\Api;

use LMVC\System\Response;

class Misc extends \LMVC\System\Controller
{
    public function __construct()
    {
        parent::__construct();

        // $flagPermission = false;

        $this->langModel = $this->model('LangModel');
        $this->LogModel = $this->model('LogModel');
        $this->settingsModel = $this->model('SettingsModel');
        $this->fileLogModel = $this->model('FileLogModel');

        $serverPort= getCgiPort();
        
        $this->customPortalModel = $this->model('CustomPortalModel');
        $this->customPortalModel->qtsApiPort = $serverPort;
        $this->qtsModel = $this->model('QTSModel');
        $this->qtsModel->qtsApiPort = $serverPort;
        if(!$this->qtsModel->isCSRFVerify()){
            $this->fileLogModel->logError('CSRF error', __FILE__);
            Response::error(403, apiFormater("Fail", 2020));
        }
        if (!$this->qtsModel->isValidUser()) {
            $this->fileLogModel->logError('You do not have permission to use this API.', __FILE__);
            Response::error(403, apiFormater('Fail', 2000));
        }
    }

    public function getLang($lang = '')
    {
        $arrLangFileList = array(
            'global',
            'submit_ticket',
            'remote_support',
            'overview',
            'settings',
            'diagnostic_tool',
            'dialog',
            'tooltip',
            'qcloud',
            'callhome',
        );
        $arrOutputLang = array();
        $arrFallbackLang = array();

        $lang = empty($lang) && isset($_COOKIE['nas_lang']) ? $_COOKIE['nas_lang'] : $lang;
        if (empty($lang)) {
            $lang = 'ENG';
        }

        $lang = strtoupper($lang);

        $langModel = new $this->langModel();
        $langModel->setLocal('ENG');

        foreach ($arrLangFileList as $langFile) {
            $langModel->loadDictionary($langFile);
            $arrFallbackLang[$langFile] = $langModel->getLangFile();
        }

        $langModel->setLocal($lang);

        foreach ($arrLangFileList as $langFile) {
            $langModel->loadDictionary($langFile);
            $arrOutputLang[$langFile] = $langModel->getLangFile();

            if (is_null($arrOutputLang[$langFile])) {
                unset($arrOutputLang[$langFile]);
            }
        }

        if (empty($arrOutputLang)) {
            $arrOutputLang = array_merge_recursive($arrFallbackLang, $arrOutputLang);
        } else {
            $arrOutputLang = array_replace_recursive($arrFallbackLang, $arrOutputLang);
        }

        Response::json($arrOutputLang);
    }

    public function getVersion()
    {
        $output=shell_exec("curl -LI ".config('customPortal_server')." -o /dev/null -w '%{http_code}' -s --max-time 8");
        Response::json(apiFormater(array(
            'version' => 'V' . APP_VERSION . ' (' . APP_BUILD . ')',
            'connection' => in_array(trim($output), [200,301,302]),
        )));
    }

    private function getAppLogString($lang = 'ENG')
    {
        $langFile = 'notification';
        $arrOutputLang = array();
        $arrFallbackLang = array();

        $lang = strtoupper($lang);

        $langModel = new $this->langModel();
        $langModel->setLocal('ENG');

        $langModel->loadDictionary('notification');
        $arrFallbackLang[$langFile] = $langModel->getLangFile();

        $langModel->setLocal($lang);

        $langModel->loadDictionary($langFile);
        $arrOutputLang[$langFile] = $langModel->getLangFile();

        if (is_null($arrOutputLang[$langFile])) {
            unset($arrOutputLang[$langFile]);
        }

        if (empty($arrOutputLang)) {
            $arrOutputLang = array_merge_recursive($arrFallbackLang, $arrOutputLang);
        } else {
            $arrOutputLang = array_replace_recursive($arrFallbackLang, $arrOutputLang);
        }

        return $arrOutputLang['notification'];
    }

    public function getAppLog($lang = '')
    {
        $lang = empty($lang) && isset($_COOKIE['nas_lang']) ? $_COOKIE['nas_lang'] : $lang;
        if (empty($lang)) {
            $lang = 'ENG';
        }

        $logKeepDays = $this->settingsModel->getMsgRetainDays();
        if ($logKeepDays == -1) {
            $logKeepDays = '';
        }

        $logs = $this->LogModel->getAppLog($logKeepDays);
        $arrAppLogString = $this->getAppLogString($lang);
        $arrAppLog = array();

        foreach ($logs as $log) {
            $logMsg = $arrAppLogString[$log['msgId']];
            $logMeta = (array) json_decode($log['msgMeta']);
            $logMeta['TOKEN_DATE'] = date('Y-m-d H:i:s', $log['createdTime']);

            if (!empty($logMeta)) {
                $logMsg = str_replace(array_keys($logMeta), array_values($logMeta), $logMsg);
            }

            array_push($arrAppLog, $logMsg);
        }

        Response::json(apiFormater(array('appLogs' => $arrAppLog)));
    }

    public function postUserGroup()
    {
        Response::json(array('nasAccount' => 'admin'));
    }
}
