<?php

namespace App\Controllers\Api;

use LMVC\System\Request;
use LMVC\System\Response;

class RemoteAccess extends \LMVC\System\Controller
{
    protected $rempteAccessModel;
    protected $logModel;

    protected $fwSKeyPath;
    protected $tmpKeyPath;
    protected $authedKeyPath = '/home/_qnap_support/.ssh/authorized_keys';
    protected $forwardSrvPort = 22;
    protected $qtsApiPort;

    public function __construct()
    {
        parent::__construct();

        $serverPort= getCgiPort();
        

        $this->remoteAccessModel = $this->model('RemoteAccessModel');
        $this->remoteAccessModel->qtsApiPort = $serverPort;
        $this->settingsModel = $this->model('SettingsModel');
        $this->fileLogModel = $this->model('FileLogModel');
        $this->remoteAccessModel->fileLogModel = $this->fileLogModel;
        $this->logModel = $this->model('LogModel');
        $this->qtsModel = $this->model('QTSModel');
        $this->qcloudModel = $this->model('QCloudModel');
        $this->remoteAccessModel->qcloudModel = $this->qcloudModel;
        $this->qtsModel->qtsApiPort = $serverPort;
        

        if(!$this->qtsModel->isCSRFVerify()){
            $this->fileLogModel->logError('CSRF error', __FILE__);
            Response::error(403, apiFormater("Fail", 2020));
        }

        if (!$this->qtsModel->isValidUser()) {
            $this->fileLogModel->logError('You do not have permission to use this API.', __FILE__);
            Response::error(403, apiFormater("Fail", 2000));
        }
        $this->fwSKeyPath = config('app_dir') . '/data/forwarding.pem';
        $this->tmpKeyPath = config('app_dir') . '/data/helpdesk_temp_key';
    }

    //{{{ public function getStatus(){
    public function getStatus()
    {
        $rs = $this->remoteAccessModel->getStatus();
        $tempAccount = $this->remoteAccessModel->getTempAccount();
        $tempId = null;
        $tempPw = null;
        $privateKey = null;
        $status = $rs['status'];
        $remoteStatus = 0;
        if (!empty($rs['ticketId'])) {
            $this->remoteAccessModel->ticketId = $rs['ticketId'];
            $remoteStatus = $this->remoteAccessModel->getRemoteSessionStatus();
        }

        if (is_array($remoteStatus)) {
            $objRStatus = json_encode($remoteStatus);
            $objRStatus = json_decode($objRStatus);

            if (isset($objRStatus->status)) {
                Response::error($objRStatus->status, apiFormater($objRStatus->msg, 2014));
            }
        }

        $closedTime = $rs['closedTime'];
        $closedMsg = '';

        if (($remoteStatus == 1 || empty($rs['status'])) && !empty($rs['ticketId'])) {
            $status = 1003;
        } else {
            $tempId = $tempAccount['tempId'];
            $tempPw = $tempAccount['tempPw'];
            $privateKey = '';
            if (file_exists($this->tmpKeyPath)) {
                $privateKey = file_get_contents($this->tmpKeyPath);
            }
            $status = $remoteStatus;
            $this->remoteAccessModel->setStatus($status, $rs['ticketId'], $rs['email']);
        }

        $this->remoteAccessModel->ticketId = $rs['ticketId'];
        $this->remoteAccessModel->email = $rs['email'];
        $this->remoteAccessModel->sshFwPort = $rs['sshForwardingPort'];
        $this->remoteAccessModel->httpFwPort = $rs['qtsForwardingPort'];
        $this->remoteAccessModel->httpsFwPort = $rs['httpsForwardingPort'];

        $logInProgressIsSet = $this->logModel->isRemoteLogSet('notification_remoteSupport_inProgress');

        if ($remoteStatus == 1002 && !$logInProgressIsSet) {
            $this->logModel->qtsLog(3);
            //$this->logModel->qtsLogInfo("Your remote support session is starting, please do not change the NAS settings during this period.");
            $this->logModel->setRemoteLog('notification_remoteSupport_inProgress');
            $this->logModel->setAppLog('notification_remoteSupport_inProgress');
        }

        if ($remoteStatus == 1004) {
            // Disable Remote Session
            $this->remoteAccessModel->disableRemoteSession();
            $this->disableSSH();

        }

        if ($remoteStatus == 1005) {
            // Disable Remote Session
            $this->remoteAccessModel->disableRemoteSession();
            $this->disableSSH();

            $closedInfo = $this->remoteAccessModel->setClosedInfo();

            if ($closedInfo == false) {

                //$this->fileLogModel->logError('Can not get remote session close info. HTTP StatusCode: ' . $srvOutput->httpStatusCode, __FILE__, __LINE__);
                $this->fileLogModel->logError('Can not get remote session close info.', __FILE__, __LINE__);
            }

            $closedMsg = $closedInfo['closedMsg'];

            $closedTime = $closedInfo['closedTime'];//date('Y/m/d H:i:s', $closedInfo['closedTime']);
            

            
            $logClosedIsSet = $this->logModel->isRemoteLogSet('notification_remoteSupport_finished');

            if (!$logClosedIsSet) {
                $this->logModel->qtsLog(4);
                //$this->logModel->qtsLogInfo("Your remote support session was finished, please check for result.");
                $this->logModel->setRemoteLog('notification_remoteSupport_finished');
                $this->logModel->setAppLog('notification_remoteSupport_finished');
            }
            
        }

        $showLog = $this->remoteAccessModel->getReportResultLog();

        Response::json(array(
            'ticketId' => $rs['ticketId'],
            'email' => $rs['email'],
            'status' => $status,
            'confirmed' => $rs['confirmedStatus'],
            'startTime' => $rs['startTime'],
            'expireTime' => $rs['expireTime'],
            'finished_time' => $closedTime,
            'login_id' => $tempId,
            'password' => $tempPw,
            'privateKey' => $privateKey,
            'finished_report' => $closedMsg,
            'extended' => $rs['extended'],
            'showLog' => $showLog,
        ));
    }
    //}}}

    //{{{ public function getKeys() {
    public function getKeys()
    {
        $supportPW = strRandom(2, 8);

        // Generate SSH key
        exec("qsh helpdeskd.genKey -v '{\"path\":\"".$this->tmpKeyPath."\"}'");
        $privateKey = file_get_contents($this->tmpKeyPath);

        $this->remoteAccessModel->qtsUserPW = $supportPW;
        $this->remoteAccessModel->setTempAccount();

        Response::json(array(
            'userId' => $this->remoteAccessModel->qtsUserID,
            'password' => $supportPW,
            'privateKey' => $privateKey,
        ));
    }
    //}}}

    //{{{ public function enable(){
    public function enable()
    {
        $ticketId = strtoupper(Request::any('ticketId')); // Bug 97640 - [Helpdesk] Remote session SSH need input password for wrong case ticket ID
        $email = Request::any('email');

        if ($ticketId === null) {
            Response::error(400, 'Missing parameter: ticketId');
        }
        if( !preg_match('/Q-[0-9]{6}-[0-9]{5}/', $ticketId) ){
            Response::error(400, 'Invalid parameter: ticketId');
        }
        if ($email === null) {
            Response::error(400, 'Missing parameter: email');
        }

        $this->remoteAccessModel->ticketId = $ticketId;
        $remoteStatus = $this->remoteAccessModel->getRemoteSessionStatus();

        if (is_array($remoteStatus)) {
            $objRStatus = json_encode($remoteStatus);
            $objRStatus = json_decode($objRStatus);

            if (isset($objRStatus->status)) {
                Response::error($objRStatus->status, apiFormater($objRStatus->msg, 2014));
            }
        }

        if ($remoteStatus == 1001 || $remoteStatus == 1002) {
            $this->fileLogModel->logDebug('Remote access case is already enabled.', __FILE__, __LINE__);
            Response::error(403, apiFormater(null, 2007));
        }

        if ($this->remoteAccessModel->isExist($ticketId, $email) && $remoteStatus == 1004) {
            $this->fileLogModel->logDebug('Remote access case is already expired.', __FILE__, __LINE__);
            Response::error(403, apiFormater(null, 2018));
        }

        if ($this->remoteAccessModel->isExist($ticketId, $email) && $remoteStatus == 1005) {
            $this->fileLogModel->logDebug('Remote access case is already closed.', __FILE__, __LINE__);
            Response::error(403, apiFormater(null, 2009));
        }

        if ($remoteStatus == 1) {
            $this->fileLogModel->logError('Remote access case is not activate.', __FILE__, __LINE__);
            Response::error(404, apiFormater(null, 2010));
        }

        //enable ssh
        $state = $this->qtsModel->sshState();
        if( $state['sshEnabled'] == 0){
            $this->qtsModel->sshEnable();
            $this->settingsModel->setNeedDisableSSH(1);
        }

        // Fix SSHD config permission
        $fixResult = $this->qtsModel->fixSSHDConfigPerm();
        if(!$fixResult){
            $this->fileLogModel->logError('No ssh config /mnt/HDA_ROOT/.config/ssh/', __FILE__, __LINE__);
            Response::error(500, apiFormater(null, 2022));
        }

        // Generate SSH Keypair
        $privateKey = file_get_contents($this->tmpKeyPath);
        $publicKey = shell_exec("/usr/bin/ssh-keygen -y -f $this->tmpKeyPath");

        $pattern = '/[\\r\\n]/';
        $pkWithTicketID = preg_replace($pattern, '', $publicKey) . ' ' . $ticketId . "\r\n";

        $this->remoteAccessModel->resetAuthedKey();

    
        

        // Check and create QTS account
        $tempAccount = $this->remoteAccessModel->getTempAccount();
        $supportID = $tempAccount['tempId'];
        $supportPW = $tempAccount['tempPw'];

        $this->remoteAccessModel->removeQTSUserByAPI();
        if ($this->remoteAccessModel->checkQTSUser()) {
            $this->remoteAccessModel->removeQTSUser();
            if ($this->remoteAccessModel->checkQTSUser()) {
                $this->fileLogModel->logError('Can not remove QTS support account.', __FILE__, __LINE__);
                Response::error(500, apiFormater(null, 2010));
            }
        }

        $this->remoteAccessModel->qtsUserPW = $supportPW;
        $po = $this->remoteAccessModel->createQTSUserByAPI();
        if (!$this->remoteAccessModel->checkQTSUser()) {
            $this->remoteAccessModel->createQTSUser();
            if (!$this->remoteAccessModel->checkQTSUser()) {
                $this->fileLogModel->logError('Can not create QTS support account.', __FILE__, __LINE__);
                Response::error(500, apiFormater(null, 2011));
            }
        }
        if(file_exists($this->authedKeyPath) || file_exists('/home/_qnap_support/.ssh')){
            exec("qsh helpdeskd.removeAuthedKey");
        }
        if (!file_exists('/home/_qnap_support/.ssh')) {
            mkdir('/home/_qnap_support/.ssh', 0777, true);
        }
        
        file_put_contents($this->authedKeyPath, $pkWithTicketID, FILE_APPEND);
        
        $base64SK = base64_encode($privateKey);

        exec("qsh helpdeskd.fixkeygen");

        //set config to close popup
        //$this->remoteAccessModel->setQTSUserConfig();

        // Get NAS http, https and ssh port
        if (empty($httpPort) && empty($sshPort)) {
            $httpPort = shell_exec("/usr/bin/qgetcfg --app=SYS_CONF 'system-web-server' 'Web Access Port' -d 80");
            $httpsPort = shell_exec("/usr/bin/qgetcfg --app=SYS_CONF 'system-web-server' 'HTTPS Port' -d 443");
            $sshPort = shell_exec("/usr/bin/qgetcfg --app=SYS_CONF 'Login' 'SSH Port' -d 22");
            $httpPort = str_replace("\n", '', $httpPort);
            $httpsPort = str_replace("\n", '', $httpsPort);
            $sshPort = str_replace("\n", '', $sshPort);
        }

        // Enable ticket
        $srvOutput = $this->remoteAccessModel->getRemoteSessionEnable($email,$supportID,$supportPW,$httpPort,$httpsPort,$sshPort,$base64SK);

        if (is_array($srvOutput)) {
            $srvOutput = json_encode($srvOutput);
        }

        $srvOutput = json_decode($srvOutput);

        if (isset($srvOutput->status) && $srvOutput->status === 'FALSE') {
            if (isset($srvOutput->result->errCode) 
                && ($srvOutput->result->errCode == 28 || $srvOutput->result->errCode == 7)
            ) {
                // Tell Server this ticket is close
                $this->remoteAccessModel->setRemoteSessionDisable();

                Response::error(408, apiFormater($srvOutput->result->errMsg, 2014));
            }
            if (isset($srvOutput->result->errCode) && $srvOutput->result->errCode == 6) {
                $this->logModel->qtsLog(8);
                $this->logModel->setAppLog('notification_remoteSupport_failed_unableToConnect');
                Response::error(500, apiFormater($srvOutput->result->errMsg, 2001));
            }
        }

        if (isset($srvOutput->httpStatusCode)) {
            $errMsg = '';
            $errCode = 0;
            $qtsErrMsg = '';

            switch ($srvOutput->httpStatusCode) {
                case '500':
                    switch ($srvOutput->result->status) {
                        case '2002':
                        case '2003':
                            if ($srvOutput->result->data->ticketId === true && $srvOutput->result->data->email === false) {
                                $errMsg = 'Invalid Email.';
                                $errCode = 2003;
                            } else {
                                $errMsg = 'Invalid ticket ID.';
                                $errCode = 2002;
                            }
                            $this->logModel->qtsLog(7);
                            $this->logModel->setAppLog('notification_remoteSupport_failed_invaildTicket');
                            break;
                        case '2004':
                            $errMsg = 'Server is too busy.';
                            $errCode = 2004;
                            $this->logModel->qtsLog(8);
                            $this->logModel->setAppLog('notification_remoteSupport_failed_unableToConnect');
                            break;
                        default:
                            $errMsg = 'Unknown Error. Status: ' . $srvOutput->result->msg;
                            $errCode = null;
                            $this->logModel->qtsLog(8);
                            $this->logModel->setAppLog('notification_remoteSupport_failed_unableToConnect');
                            break;
                    }

                    $this->fileLogModel->logError($errMsg, __FILE__, __LINE__);
                    $this->removeQTSUser();
                    Response::error(500, apiFormater(null, $errCode));
                    break;
                case '404':
                    $errMsg = 'Cannot connect to the server';
                    $errCode = 2001;
                    $this->logModel->qtsLog(8);
                    $this->logModel->setAppLog('notification_remoteSupport_failed_unableToConnect');

                    $this->fileLogModel->logError($errMsg, __FILE__, __LINE__);
                    $this->removeQTSUser();
                    Response::error(500, apiFormater(null, $errCode));
                    break;
                default:
                    break;
            }
        }

        $status = $srvOutput->status;

        $expireTime = $srvOutput->data->expired_time;
        $forwardIP = $srvOutput->data->ip;
        $forwardHttpPort = $srvOutput->data->http_port;
        $forwardHttpsPort = $srvOutput->data->https_port;
        $forwardSSHPort = $srvOutput->data->ssh_port;

        $date = new \DateTime("{$expireTime}");
        $date->sub(new \DateInterval('P7D'));
        $startTime = $date->format('Y-m-d H:i:s');

        // Generate and send SSH Keypair for forwarding server
        $this->remoteAccessModel->email=$email;
        $this->remoteAccessModel->fwSKeyPath = $this->fwSKeyPath;
        $srvOutput = $this->remoteAccessModel->generateFwKeypair();

        if (is_array($srvOutput) && isset($srvOutput->status) && $srvOutput->status === 'FALSE') {
            $srvOutput = json_encode($srvOutput);
            $srvOutput = json_decode($srvOutput);

            if (isset($srvOutput->result->errCode) && $srvOutput->result->errCode == 28) {
                // Tell Server this ticket is close
                $this->remoteAccessModel->setRemoteSessionDisable();

                $this->removeQTSUser();
                Response::error(500, apiFormater($srvOutput->result->errMsg, 2014));
            }
        }

        sleep(1);

        $test_ssh_connection = shell_exec('ssh -o ConnectTimeout=3 ubuntu@'.config('helpdesk_server').' -p 22 2>&1');
        if (strpos($test_ssh_connection, 'timed out') !== false) {
            $this->remoteAccessModel->setRemoteSessionDisable();
            $this->remoteAccessModel->removeFwSSHPKey();

            $this->fileLogModel->logError('SSH tunnel connection timeout', __FILE__, __LINE__);
            $this->removeQTSUser();
            Response::error(500, apiFormater('SSH tunnel connection timeout', 2021));
        }

        // Establish SSH tunnel
        $this->remoteAccessModel->sshPort = $sshPort;
        $this->remoteAccessModel->httpPort = $httpPort;
        $this->remoteAccessModel->httpsPort = $httpsPort;
        $this->remoteAccessModel->forwardIP = $forwardIP;
        $this->remoteAccessModel->forwardSrvPort = $this->forwardSrvPort;

        $this->remoteAccessModel->sshFwPort = $forwardSSHPort;
        $this->remoteAccessModel->establishSSHFwTunnel();

        $this->remoteAccessModel->httpFwPort = $forwardHttpPort;
        $this->remoteAccessModel->establishHTTPFwTunnel();

        $this->remoteAccessModel->httpsFwPort = $forwardHttpsPort;
        $this->remoteAccessModel->establishHTTPSFwTunnel();

        // Check SSH tunnel is established, if not, try to reestablish the tunnel
        $isTunnelEstablished = false;
        $ctrEstablishTunnel = 0;
        $ctrFailedTunnel = 3;

        $r1=[];
        $r2=[];
        $r3=[];

        while ($ctrEstablishTunnel < 3) {
            // Dirty way... waiting for process established
            sleep(1);

            $pidArray = $this->remoteAccessModel->getSSHTunnelPid();
            array_push($r1, $pidArray);
            if ($pidArray !== false && !empty($pidArray['sshFwPid'])) {
                --$ctrFailedTunnel;
                $ctrEstablishTunnel = 0;
                break;
            }

            $this->remoteAccessModel->establishSSHFwTunnel();
            ++$ctrEstablishTunnel;
        }

        while ($ctrEstablishTunnel < 3) {
            sleep(1);

            $pidArray = $this->remoteAccessModel->getSSHTunnelPid();
            array_push($r2, $pidArray);
            if ($pidArray !== false && !empty($pidArray['httpFwPid'])) {
                --$ctrFailedTunnel;
                $ctrEstablishTunnel = 0;
                break;
            }

            $this->remoteAccessModel->establishHTTPFwTunnel();
            ++$ctrEstablishTunnel;
        }

        while ($ctrEstablishTunnel < 3) {
            sleep(1);

            $pidArray = $this->remoteAccessModel->getSSHTunnelPid();
            array_push($r3, $pidArray);
            if ($pidArray !== false && !empty($pidArray['httpsFwPid'])) {
                --$ctrFailedTunnel;
                $ctrEstablishTunnel = 0;
                break;
            }

            $this->remoteAccessModel->establishHTTPSFwTunnel();
            ++$ctrEstablishTunnel;
        }

        if ($ctrFailedTunnel >= 2) {
            // Tell Server this ticket is close
            $this->remoteAccessModel->setRemoteSessionDisable();
            $this->remoteAccessModel->removeFwSSHPKey();

            $this->fileLogModel->logError('Can not establish reverse SSH tunnel connection. Failed Count:' . $ctrFailedTunnel, __FILE__, __LINE__);
            $this->removeQTSUser();
            Response::error(500, apiFormater(array(
                'Failed Tunnel Count' => $ctrFailedTunnel,
                $r1,$r2,$r3,
                [
                    $this->remoteAccessModel->httpPort,
                    $this->remoteAccessModel->httpsPort,
                ],
                [
                    $this->remoteAccessModel->httpFwPort,
                    $this->remoteAccessModel->httpsFwPort,
                ],
                [
                    $this->remoteAccessModel->sshFwPort,
                    $this->remoteAccessModel->sshPort,
                    $this->remoteAccessModel->forwardIP,
                    $this->remoteAccessModel->forwardSrvPort,
                    $this->remoteAccessModel->fwSKeyPath
                ]    
            ), 2005));
        }

        

        $success = $this->remoteAccessModel->enable($ticketId, $email, $startTime, $expireTime, $forwardIP, $forwardSSHPort, $forwardHttpPort, $forwardHttpsPort);

        // Launch WatchDog
        $watchDog = new \App\Controllers\Cli\WatchDog();
        $bark = $watchDog->liveOrDie();

        // Enable work screen
        $this->remoteAccessModel->setWorkScreen(1);
        $this->logModel->qtsLog(5,$ticketId,$startTime,$expireTime);
        //$this->logModel->qtsLogInfo("You have enable remote support with #{$ticketId} from {$startTime} to {$expireTime}.");
        $this->logModel->setAppLog('notification_remoteSupport_success', array('TOKEN_TICKET' => "#{$ticketId}", 'TOKEN_START_DATE' => "{$startTime}", 'TOKEN_END_DATE' => "{$expireTime}"));
        
        $this->remoteAccessModel->enableReportResultLog();

        Response::json(array(
            'ticketId' => $ticketId,
            'email' => $email,
            'startTime' => $startTime,
            'expireTime' => $expireTime,
            'failedTunnel' => $ctrFailedTunnel
        ));
    }
    //}}}

    private function removeQTSUser(){
        $this->remoteAccessModel->removeQTSUserByAPI();
        if ($this->remoteAccessModel->checkQTSUser()) {
            $this->remoteAccessModel->removeQTSUser();
            if ($this->remoteAccessModel->checkQTSUser()) {
                $this->fileLogModel->logError('Can not remove QTS support account.', __FILE__, __LINE__);
                Response::error(500, apiFormater(null, 2010));
            }
        }
    }

    //{{{ public function disable(){
    public function disable()
    {
        $ticketId = strtoupper(Request::any('ticketId'));
        $email = Request::any('email');
        $tempAccount = $this->remoteAccessModel->getTempAccount();
        $supportID = $tempAccount['tempId'];

        if ($ticketId === null) {
            Response::error(400, 'Missing parameter: ticketId');
        }

        if ($email === null) {
            Response::error(400, 'Missing parameter: email');
        }

        $this->remoteAccessModel->ticketId = $ticketId;
        $this->remoteAccessModel->email = $email;

        $remoteStatus = $this->remoteAccessModel->getRemoteSessionStatus();

        if (is_array($remoteStatus)) {
            $objRStatus = json_encode($remoteStatus);
            $objRStatus = json_decode($objRStatus);

            if (isset($objRStatus->status)) {
                Response::error($objRStatus->status, apiFormater($objRStatus->msg, 2014));
            }
        }

        if ($this->remoteAccessModel->isExist($ticketId, $email) && ($remoteStatus == 1000 || $remoteStatus == 1003 || $remoteStatus == 1)) {
            $this->fileLogModel->logDebug('Remote access case is already disabled.', __FILE__, __LINE__);
            Response::error(403, apiFormater(null, 2008));
        }

        if ($this->remoteAccessModel->isExist($ticketId, $email) && $remoteStatus == 1004) {
            $this->fileLogModel->logDebug('Remote access case is already expired.', __FILE__, __LINE__);
            Response::error(403, apiFormater(null, 2018));
        }

        if ($this->remoteAccessModel->isExist($ticketId, $email) && $remoteStatus == 1005) {
            $this->remoteAccessModel->ticketId = $ticketId;
            $remoteInfo = $this->remoteAccessModel->getRemoteSessionInfo();
            $finishedTime = strtotime($remoteInfo->data->finished_time);
            $this->fileLogModel->logDebug('Remote access case is already closed.', __FILE__, __LINE__);
            Response::error(403, apiFormater(array('finished_time' => date('Y/m/d H:i:s', $finishedTime)), 2009));
        }

        $forwardPorts = $this->remoteAccessModel->getForwardPort($ticketId);

        $forwardPort = $forwardPorts[0];
        $sshFwPort = $forwardPort['sshForwardingPort'];
        $httpFwPort = $forwardPort['qtsForwardingPort'];
        $httpsFwPort = $forwardPort['httpsForwardingPort'];
        $forwardIP = $forwardPort['forwardingServer'];

        // Kill Reverse SSH tunnel
        $this->remoteAccessModel->sshFwPort = $sshFwPort;
        $this->remoteAccessModel->httpFwPort = $httpFwPort;
        $this->remoteAccessModel->httpsFwPort = $httpsFwPort;
        $this->remoteAccessModel->killSSHTunnel();

        // Remove SSH publickey from forwarding server
        $this->remoteAccessModel->removeFwSSHPKey();

        // Restore .ssh/authorized_keys
        $this->remoteAccessModel->resetAuthedKey();
        $this->remoteAccessModel->restoreAuthedKey();

        // Remove support account
        $this->remoteAccessModel->removeQTSUserByAPI();
        if ($this->remoteAccessModel->checkQTSUser()) {
            $this->remoteAccessModel->removeQTSUser();
        }

        // Tell Server this ticket is close
        $setRemoteDisable = $this->remoteAccessModel->setRemoteSessionDisable();
        if ($setRemoteDisable !== true) {
            $this->remoteAccessModel->ticketId = $ticketId;
            $remoteInfo = $this->remoteAccessModel->getRemoteSessionInfo();
            $finishedTime = strtotime($remoteInfo->data->finished_time);
            $this->fileLogModel->logError('Can not disable remote session.', __FILE__, __LINE__);
            Response::error(500, apiFormater(array('finished_time' => date('Y/m/d H:i:s', $finishedTime)), 2009));
        }else{
            $this->logModel->setRemoteLog('notification_remoteSupport_disabled');
        }

        //disable ssh if ssh open by helpdesk
        $this->disableSSH();


        // Set local db
        $this->remoteAccessModel->setDisable();

        // Kill WatchDog (err ahh!)
        $watchDog = new \App\Controllers\Cli\WatchDog();
        $bark = $watchDog->liveOrDie();

        // Disable work screen
        $this->remoteAccessModel->setWorkScreen(0);

        Response::json(array(
            'ticketId' => $ticketId,
            'status' => '1003',
        ));
    }
    //}}}

    //{{{ public function extend(){
    public function extend()
    {
        $ticketId = Request::any('ticketId');
        $email = Request::any('email');

        if ($ticketId === null) {
            Response::error(400, 'Missing parameter: ticketId');
        }

        if ($email === null) {
            Response::error(400, 'Missing parameter: email');
        }

        $this->remoteAccessModel->ticketId = $ticketId;
        $this->remoteAccessModel->email = $email;
        $remoteStatus = $this->remoteAccessModel->getRemoteSessionStatus();

        if (is_array($remoteStatus)) {
            $objRStatus = json_encode($remoteStatus);
            $objRStatus = json_decode($objRStatus);

            if (isset($objRStatus->status)) {
                Response::error($objRStatus->status, apiFormater($objRStatus->msg, 2014));
            }
        }

        if ($this->remoteAccessModel->isExist($ticketId, $email) && ($remoteStatus == 1000 || $remoteStatus == 1003 || $remoteStatus == 1)) {
            $this->fileLogModel->logDebug('Remote access case is already disabled.', __FILE__, __LINE__);
            Response::error(403, apiFormater(null, 2008));
        }

        if ($this->remoteAccessModel->isExist($ticketId, $email) && $remoteStatus == 1004) {
            $this->fileLogModel->logDebug('Remote access case is already expired.', __FILE__, __LINE__);
            Response::error(403, apiFormater(null, 2018));
        }

        if ($this->remoteAccessModel->isExist($ticketId, $email) && $remoteStatus == 1005) {
            $this->remoteAccessModel->ticketId = $ticketId;
            $remoteInfo = $this->remoteAccessModel->getRemoteSessionInfo();
            $finishedTime = strtotime($remoteInfo->data->finished_time);
            $this->fileLogModel->logDebug('Remote access case is already closed.', __FILE__, __LINE__);
            Response::error(403, apiFormater(array('finished_time' => date('Y/m/d H:i:s', $finishedTime)), 2009));
        }

        if ($this->remoteAccessModel->isExtended($ticketId, $email)) {
            $expireTime = $this->remoteAccessModel->getExpireTime($ticketId, $email);
            $this->fileLogModel->logError('Can not extend ticket expire time.', __FILE__, __LINE__);
            Response::error(500, apiFormater($expireTime, 2013));
        }

        $curlUrl = 'https://' . config('helpdesk_server') . "/apps/qdesk_service/api/v1/qpkg/remote/{$ticketId}/delayexpired";
        $srvOutput = curlWrapper('GET', $curlUrl);
        if (is_array($srvOutput)) {
            $srvOutput = json_encode($srvOutput);
        }
        $srvOutput = json_decode($srvOutput);

        if (isset($srvOutput->httpStatusCode) && $srvOutput->httpStatusCode === 500) {
            $this->fileLogModel->logError('Can not extend ticket expire time.', __FILE__, __LINE__);
            Response::error(500, apiFormater(null, 2013));
        }

        $expireTime = $srvOutput->data->expired_time;

        if ($expireTime === null) {
            $expireTime = $this->remoteAccessModel->getExpireTime($ticketId, $email);
            $this->fileLogModel->logError('Can not extend ticket expire time.', __FILE__, __LINE__);
            Response::error(500, apiFormater($expireTime, 2013));
        } else {
            $this->remoteAccessModel->setExpireTime($ticketId, $email, $expireTime);
            $this->logModel->qtsLog(6,$expireTime);
            //$this->logModel->qtsLogInfo("Your remote support session is been extended to {$expireTime}.");
            $this->logModel->setAppLog('notification_remoteSupport_extended', array('TOKEN_EXPIRED_DATE' => "{$expireTime}"));

            Response::json(array(
                'ticketId' => $ticketId,
                'expireTime' => $expireTime,
            ));
        }
    }
    //}}}

    //{{{ public function confirm() {
    public function confirm()
    {
        $ticketId = Request::any('ticketId');
        $email = Request::any('email');

        if ($ticketId === null) {
            Response::error(400, 'Missing parameter: ticketId');
        }

        if ($email === null) {
            Response::error(400, 'Missing parameter: email');
        }

        $this->remoteAccessModel->confirmFinished($ticketId, $email);

        Response::json(array(
            'ticketId' => $ticketId,
            'confirmed' => 1,
        ));
    }
    //}}}

    public function sshStatus()
    {
        $result = $this->qtsModel->sshState();
        if($result){
            Response::json($result);
        }else{
            Response::error(500, apiFormater(null, 2019));
        }
    }

    public function disableSSH(){
        //disable ssh if ssh open by helpdesk
        if($this->settingsModel->getNeedDisableSSH()==1){
            $this->qtsModel->sshDisable();
            $this->settingsModel->setNeedDisableSSH(0);
        }
    }

    public function disableReportResultLog(){
        $this->remoteAccessModel->disableReportResultLog();
        Response::json("done");
    }

}
