<?php

namespace App\Controllers\Api;

use LMVC\System\Request;
use LMVC\System\Response;

class Settings extends \LMVC\System\Controller
{
    protected $settingsModel;

    public function __construct()
    {
        parent::__construct();
        $serverPort= getCgiPort();


        $this->settingsModel = $this->model('SettingsModel');
        $this->fileLogModel = $this->model('FileLogModel');
        $this->qtsModel = $this->model('QTSModel');
        $this->LogModel = $this->model('LogModel');
        $this->qcloudModel = $this->model('QCloudModel');
        $this->qtsModel->qtsApiPort = $serverPort;
        
        $this->remoteAccessModel = $this->model('RemoteAccessModel');
        $this->remoteAccessModel->qtsApiPort = $serverPort;
        $this->remoteAccessModel->fileLogModel = $this->fileLogModel;

        $this->customPortalModel = $this->model('CustomPortalModel');
        $this->customPortalModel->qtsApiPort = $serverPort;


        if(!$this->qtsModel->isCSRFVerify()){
            $this->fileLogModel->logError('CSRF error', __FILE__);
            Response::error(403, apiFormater("Fail", 2020));
        }
        if (!$this->qtsModel->isValidUser()) {
            $this->fileLogModel->logError('You do not have permission to use this API.', __FILE__);
            Response::error(403, apiFormater("Fail", 2000));
        }
       
    }

    public function getMsgRetainDays()
    {
        $keepDays = $this->settingsModel->getMsgRetainDays();

        return array('keepDays' => $keepDays);
    }

    public function setMsgRetainDays($keepDays)
    {
        $this->settingsModel->setMsgRetainDays($keepDays);

        return array('keepDays' => $keepDays);
    }

    public function getLocation()
    {
        $location = $this->settingsModel->getLocation();

        if (empty($location)) {
            Response::error(500, 'Cannot get settings.');
        }

        return $location;
    }

    public function setLocation($locRegion, $locArea)
    {
        $this->settingsModel->setLocation($locRegion, $locArea);

        return array('region' => $locRegion, 'area' => $locArea);
    }

    public function getAttachSyslog()
    {
        $needAttach = $this->settingsModel->getAttachSyslog();

        return array('needAttach' => $needAttach);
    }
    
    public function getNeedDisableSSH()
    {
        $needDisableSSH = $this->settingsModel->getNeedDisableSSH();

        return array('needDisableSSH' => $needDisableSSH);
    }


    public function setAttachSyslog($needAttach)
    {
        $this->settingsModel->setAttachSyslog($needAttach);

        return array('needAttach' => $needAttach);
    }

    public function getUpdateWarn()
    {

        $srvOutput = $this->customPortalModel->getUpdateWarn();
        if(is_array($srvOutput)){
            if(isset($result['status'])&&$result['status'] == "FALSE"){
                return array('update'=>0,'errorCode'=>0);
            }else{
                return array('update'=>0,'errorCode'=>1);
            }
        }else{
            $result = json_decode($srvOutput); 
            if(isset($result->force_update)&&$result->force_update){
                $updateWarn = $this->settingsModel->getUpdateWarn();
                return array('times' => $updateWarn,'update'=>1, 'response'=>$result);
            }else{
                return array('update'=>0,'response'=>$result);
            }
        }
    }

    public function setUpdateWarn($updateWarn)
    {
        $this->settingsModel->setUpdateWarn($updateWarn);

        return array('updateWarn' => $updateWarn);
    }

    

    public function getSettings($type = null)
    {
        switch ($type) {
            case 'showguides':
                Response::json(apiFormater($this->settingsModel->getShowGuides()));
                break;
            case 'keepmsg':
                Response::json($this->getMsgRetainDays());
                break;
            case 'attachlog':
                Response::json($this->getAttachSyslog());
                break;
            case 'updatewarn':
                Response::json($this->getUpdateWarn());
                break;
            default:
                $rs = $this->remoteAccessModel->getStatus();
                $this->remoteAccessModel->sshFwPort = $rs['sshForwardingPort'];
                $this->remoteAccessModel->httpFwPort = $rs['qtsForwardingPort'];
                $this->remoteAccessModel->httpsFwPort = $rs['httpsForwardingPort'];
                $arrSettings = array_merge(
                    $this->getMsgRetainDays(),
                    $this->getAttachSyslog(),
                    $this->getNeedDisableSSH()
                );
                $testConnection = $this->customPortalModel->getCustomer();
                if(isset($testConnection['status'])&&$testConnection['status']==408)$testConnection = false;
                if(is_array($testConnection)&&count($testConnection)>0&&isset($testConnection[0]->API_KEY))$testConnection = true;
                $arrSettings['connection'] = $testConnection;
                $arrSettings['cpUrl']="https://".config('customPortal_server')."/";
                $arrSettings['qidUrl']="https://".config('accountCenter_server')."/";
                Response::json($arrSettings);
                break;
        }
    }

    public function setSettings($type = null)
    {
        $isShowGuides = Request::any('isShowGuides');
        $keepDays = Request::any('keepDays');
        $needAttach = Request::any('needAttach');
        $updateWarn = Request::any('updateWarn');

        switch ($type) {
            case 'showguides':
                Response::json(apiFormater($this->settingsModel->setShowGuides($isShowGuides)));
                break;
            case 'keepmsg':
                Response::json($this->setMsgRetainDays($keepDays));
                break;
            case 'attachlog':
                Response::json($this->setAttachSyslog($needAttach));
                break;
            case 'updatewarn':
                Response::json($this->setUpdateWarn($updateWarn));
                break;
            default:
                $arrSettings = array_merge(
                    $this->setMsgRetainDays($keepDays),
                    $this->setAttachSyslog($needAttach));
                Response::json($arrSettings);
                break;
        }
    }

    public function getQidInfo(){
        $curuser = Request::any('curuser');
        //if( !$this->qtsModel->isAdmin() ){
        //    Response::error(500, 'Only administrator can login.');
        //}else{
        $userinfo = $this->qcloudModel->user_info($curuser);
        if(is_array($userinfo)){
            $userinfo=json_encode($userinfo);
            $userinfo=json_decode($userinfo);
        }
        if(isset($userinfo->status)){
            if($userinfo->status==3005)$this->logoutQID();
            else Response::error(200, apiFormater($userinfo->result,$userinfo->status));
        }else{
            if(isset($userinfo->result) && $userinfo->message=='OK.'){
                $result = [
                    $userinfo,
                    'last_name'=> $userinfo->result->last_name ? $userinfo->result->last_name : '',
                    'first_name'=>$userinfo->result->first_name ? $userinfo->result->first_name : '',
                    'user_id'=>$userinfo->result->user_id,
                    'phones'=> $userinfo->result->phones,
                    'emails'=> $userinfo->result->emails,
                    'company'=> isset($userinfo->result->company)&&$userinfo->result->company ? $userinfo->result->company : '',
                    'country'=> isset($userinfo->result->country)&&$userinfo->result->country ? $userinfo->result->country : '',
                    'timezone'=>isset($userinfo->result->timezone)&&$userinfo->result->timezone ? $userinfo->result->timezone : [],
                    ];

                Response::json($result); 
            }else{
                Response::error(403, apiFormater($userinfo->result,3002));
            }
            
        }
    }
    public function bindQidInfo(){
        $curuser = Request::any('curuser');
        if( !$this->qtsModel->isAdmingroup() ){
            Response::error(500, 'Only administrator can login.');
        }else{
            $userinfo = $this->qcloudModel->bind_user_info($curuser);
            if(is_array($userinfo)){
                $userinfo=json_encode($userinfo);
                $userinfo=json_decode($userinfo);
            }
            if(isset($userinfo->status)){
                if($userinfo->status==3005)$this->logoutQID();
                else Response::error(200, apiFormater($userinfo->result,$userinfo->status));
            }else{
                if(isset($userinfo->result) && $userinfo->message=='OK.'){
                    $result = [
                        //$userinfo,
                        //'last_name'=> $userinfo->result->last_name ? $userinfo->result->last_name : '',
                        //'first_name'=>$userinfo->result->first_name ? $userinfo->result->first_name : '',
                        'user_id'=>$userinfo->result->user_id,
                        'phones'=> $userinfo->result->phones,
                        'emails'=> $userinfo->result->emails,
                        //'company'=> isset($userinfo->result->company)&&$userinfo->result->company ? $userinfo->result->company : '',
                        //'country'=> isset($userinfo->result->country)&&$userinfo->result->country ? $userinfo->result->country : '',
                        //'timezone'=>isset($userinfo->result->timezone)&&$userinfo->result->timezone ? $userinfo->result->timezone : [],
                        ];

                    Response::json($result); 
                }else{
                    Response::error(403, apiFormater($userinfo->result,3002));
                }
                
            }
        }
    }

    public function loginQID(){
        $username = Request::any('username');
        $password = Request::any('passwd');
        $curuser = Request::any('curuser');
        $loginResult = $this->qcloudModel->login($username,$password,$curuser);
        if (is_array($loginResult)) {
            $objRStatus = json_encode($loginResult);
            $objRStatus = json_decode($objRStatus);

            if (isset($objRStatus->status)) {
                Response::error($objRStatus->status, apiFormater($objRStatus->msg, 3000));
            }
        }else{
            if(isset($loginResult->access_token)){
                $this->LogModel->setAppLog('notification_qid_login', array("\${qid}" => $this->maskQid($username)));
                Response::json(["result"=>"success","access_token"=>true]);
            }else{
                Response::json($loginResult);
            }
            
            //Response::json(["result"=>"success","access_token"=>true]);
        }
    }

    public function logoutQID(){
        $logoutResult = $this->qcloudModel->logout();
        $this->LogModel->setAppLog('notification_qid_logout', array("\${qid}" => $this->maskQid($logoutResult->account) ));
        Response::json(["message"=>"logout qid"]);
        
    }
    public function bindQID(){
        $username = Request::any('username');
        $password = Request::any('passwd');
        $curuser = Request::any('curuser');
        $loginResult = $this->qcloudModel->bind($username,$password,$curuser);
        if (is_array($loginResult)) {
            $objRStatus = json_encode($loginResult);
            $objRStatus = json_decode($objRStatus);

            if (isset($objRStatus->status)) {
                Response::error($objRStatus->status, apiFormater($objRStatus->msg, 3000));
            }
        }else{
            if(isset($loginResult->access_token)){
                $this->LogModel->setAppLog('notification_qid_login', array("\${qid}" => $this->maskQid($username)));

            }
            Response::json(["result"=>"success","access_token"=>true]);
        }
    } 
    public function unbindQID(){
        $logoutResult = $this->qcloudModel->unbind();
        $this->LogModel->setAppLog('notification_qid_logout', array("\${qid}" => $this->maskQid($logoutResult->account) ));
        $this->settingsModel->setCallHomeService(false);
        chmod('/mnt/ext/opt/qdesk/shell/disableCallHome.sh', 0755);
        shell_exec('/mnt/ext/opt/qdesk/shell/disableCallHome.sh'); 
        $this->settingsModel->setCallHomeLog(true);
        $this->fileLogModel->logDebug('Call Home disable.', __FILE__, __LINE__);
        Response::json([
            "result"=>"success",
            "emails"=>[],
            "phones"=>[],
            "user_id"=>""
        ]);  
    }

    private function maskQid($username){
        $mask = preg_split("/@/", $username );
        if(count($mask)>1)
            return $this->mask_email($username);
        else 
            return $this->mask_phone($username);
    }

    private function mask_email($email)
    {
        list( $user, $domain ) = preg_split("/@/", $email ); 

        $len = strlen( $user ); 
        $offset=1;
        $mask_count = $len-$offset*2;
        $masked = substr( $user, 0, $offset ) 
                .str_repeat( "*", $mask_count ) 
                .substr( $user, $len-$offset ); 

        return( $masked.'@'.$domain ); 
    }
    private function mask_phone($phone)
    {
        $len = strlen( $phone ); 
        $offset=2;
        $mask_count = $len-$offset*2;
        $masked = substr( $phone, 0, $offset ) 
                .str_repeat( "*", $mask_count ) 
                .substr( $phone, $len-$offset ); 

        return $masked ; 
    }

    public function refreshToken(){
        $_SESSION['token']=null;
        $this->qtsModel->getCSRFtoken();
        Response::json(["csrf"=>$_SESSION['token'],"expired"=>$_SESSION['token_expired']]);
    }


}
