<?php

namespace App\Controllers\Cli;

class DbUtils extends \LMVC\System\Controller
{
    protected $dbUtils;

    public function __construct()
    {
        parent::__construct();

        $this->dbUtils = $this->model('DbUtilsModel');
        $this->fileLogModel = $this->model('FileLogModel');

        // if (strtolower(php_sapi_name()) !== 'cli') {
        //     $this->fileLogModel->logError('You can not use this function via web.', __FILE__);
        //     die('You can not use this function via web. File: ' . __FILE__);
        // }
    }

    public function migrate()
    {
        $dbConfig = config('database');
        $dbFilePath = $dbConfig['database_filepath'];

        $currSchemaVersion = -1;

        if (file_exists($dbFilePath)) {
            echo "DB file exist\n";
            $schemaVersion = $this->dbUtils->getSchemaVersion();
            if (isset($schemaVersion['cfgValue'])) {
                if ($schemaVersion['cfgValue'] < DB_SCHEMA_VERSION) {
                    echo "Current schema version: " . DB_SCHEMA_VERSION . "\n";
                    echo "DB Schema version: {$schemaVersion['cfgValue']}, do migration\n";

                    $this->fileLogModel->logDebug("[Migration] Current schema version: " . DB_SCHEMA_VERSION . " > DB Schema version: {$schemaVersion['cfgValue']}, do migration");
                    $currSchemaVersion = $schemaVersion['cfgValue'] + 1;
                } else {
                    echo "DB does not need to migrate.\n";
                    $currSchemaVersion = -1;
                }
            } else {
                echo "Schema version not found. => ";
                $this->fileLogModel->logDebug('[Migration] Schema version not found');

                if ($this->dbUtils->isValidDB()) {
                    echo "Valid DB found => Do migration\n";
                    $this->fileLogModel->logDebug('[Migration] Valid DB found => Do migration');
                    $currSchemaVersion = 1;
                } else {
                    echo "Broken DB file => Do create\n";
                    $this->fileLogModel->logDebug('[Migration] Broken DB file => Do create');
                    unlink($dbFilePath);
                    $this->dbUtils->initDB();
                    $currSchemaVersion = 0;
                }
            }
        } else {
            echo "DB file does not exist\n";
            $this->fileLogModel->logDebug('[Migration] DB file does not exist.');
            $currSchemaVersion = 0;
        }

        switch ($currSchemaVersion) {
            case '0':
                echo "0. Create DB... ";
                if ($this->dbUtils->migrateInitDB() === false) {
                    unlink($dbFilePath);
                    echo "Failed!\n";
                    $this->fileLogModel->logError('[Migration][0] Failed.');
                    break;
                }
                echo "Done!\n";
                $this->fileLogModel->logDebug('[Migration][0] Successful.');
                // Finish initial DB
            case '1':
                echo "1. Migrate DB schema to version 1... ";
                // Backup exist db for migrate fail rollback
                $dbFileBackup = $dbFilePath . '_bak';
                copy($dbFilePath, $dbFileBackup);

                // Do migration
                if ($this->dbUtils->migrateVer1() === false) {
                    rename($dbFileBackup, $dbFilePath);
                    echo "Failed!\n";
                    $this->fileLogModel->logError('[Migration][1] Failed.');
                    break;
                }
                unlink($dbFileBackup);
                echo "Done!\n";
                $this->fileLogModel->logDebug('[Migration][1] Successful. Set DB schema version to 1');
                // Finish migrate to version 1
            case '2':
                echo "2. Migrate DB schema to version 2... ";
                // Backup exist db for migrate fail rollback
                $dbFileBackup = $dbFilePath . '_bak';
                copy($dbFilePath, $dbFileBackup);

                // Do migration
                if ($this->dbUtils->migrateVer2() === false) {
                    rename($dbFileBackup, $dbFilePath);
                    echo "Failed!\n";
                    $this->fileLogModel->logError('[Migration][2] Failed.');
                    break;
                }
                unlink($dbFileBackup);
                echo "Done!\n";
                $this->fileLogModel->logDebug('[Migration][2] Successful. Set DB schema version to 2');
                // Finish migrate to version 2
            case '3':
                echo "3. Migrate DB schema to version 3... ";
                // Backup exist db for migrate fail rollback
                $dbFileBackup = $dbFilePath . '_bak';
                copy($dbFilePath, $dbFileBackup);

                // Do migration
                if ($this->dbUtils->migrateVer3() === false) {
                    rename($dbFileBackup, $dbFilePath);
                    echo "Failed!\n";
                    $this->fileLogModel->logError('[Migration][3] Failed.');
                    break;
                }
                unlink($dbFileBackup);
                echo "Done!\n";
                $this->fileLogModel->logDebug('[Migration][3] Successful. Set DB schema version to 3');
                // Finish migrate to version 3
        }
    }
}
