<?php
namespace App\Controllers\Cli;
class SupportUtils extends \LMVC\System\Controller
{
    protected $remoteAccessModel;
    protected $uploadFileSizeLimit;
    protected $uploadFileAmountLimit;
    public function __construct()
    {
        parent::__construct();
        $this->remoteAccessModel = $this->model('RemoteAccessModel');
        $this->fileLogModel = $this->model('FileLogModel');
        // if (strtolower(php_sapi_name()) !== 'cli') {
        //     $this->fileLogModel->logError('You can not use this function via web.', __FILE__);
        //     die('You can not use this function via web. File: ' . __FILE__);
        // }
        $this->uploadFileSizeLimit = 52428800; // 50MB
        $this->uploadFileAmountLimit = 10;
    }

    public function upload($file)
    {
        $fileTotal = 0;
        $rs = $this->remoteAccessModel->getStatus();
        $ticketId = $rs['ticketId'];
        if ($rs['status'] == 1001 || $rs['status'] == 1002) {
            $file = trim($file);
            $filePath = '';
            $arrCurlFiles = array();
            if (strpos($file, ',') !== false) {
                $file = explode(',', $file);
                unset($file[0]);
                $fileTotal = count($file);
            }
            if ($fileTotal > $this->uploadFileAmountLimit) {
                printf("\r\n[%s] You can only upload %d files at one time.\r\n", colorize('ERROR', 'ERROR'), $this->uploadFileAmountLimit);
                die();
            }
            if ($fileTotal === 0) {
                printf("\r\n[%s] No files to upload.\r\n", $ticketId);
                die();
            }
            printf("\r\n[%s] Send Files to Support Center\r\n\r\n", $ticketId);
            $fileCount = 0;
            if (is_array($file)) {
                foreach ($file as $f) {
                    $f = str_replace('"', '', $f);
                    $filePath = realpath($f);
                    if (empty($filePath)) {
                        printf("[%s] %s -> [%s]\r\n", $ticketId, $f, colorize('File Not Found', 'ERROR'));
                        continue;
                    }
                    $fileSize = filesize($filePath);
                    if ($fileSize > $this->uploadFileSizeLimit) {
                        printf("[%s] %s (%s) -> [%s]\r\n", $ticketId, $filePath, sizeConvert($fileSize), colorize('File Too Large', 'WARN'));
                        continue;
                    }
                    $supportUtils = $this->model('SupportUtilsModel');
                    $supportUtils->qcloudModel = $this->model('QCloudModel');
                    $supportUtils->ticketId = $ticketId;
                    $supportUtils->filePath = $filePath;
                    $uploadResult = $supportUtils->uploadFile();
                    if ($uploadResult->result === "Success") {
                        printf("[%s] %s (%s) -> [%s]\r\n", $ticketId, $filePath, sizeConvert($fileSize), colorize('OK', 'SUCCESS'));
                        ++$fileCount;
                    } else {
                        if (isset($uploadResult->status) && $uploadResult->status == 3000) {
                            printf("[%s] %s -> [%s]\r\n", $ticketId, $f, colorize('No QID access token', 'ERROR'));
                        } else if(isset($uploadResult->result->errCode) && $uploadResult->result->errCode == 28){
                            printf("[%s] %s -> [%s]\r\n", $ticketId, $f, colorize('Request Timeout', 'ERROR'));
                        } else {
                            printf("[%s] %s -> [%s]\r\n", $ticketId, $f, colorize($uploadResult->httpStatusCode, 'ERROR'));
                        }
                        continue;
                    }
                }
            } else {
                $filePath = str_replace('"', '', realpath($file));
                if (empty($filePath)) {
                    printf("[%s] %s -> [%s]\r\n", $ticketId, $f, colorize('File Not Found', 'ERROR'));
                } else {
                    $fileSize = filesize($filePath);
                    if ($fileSize > $this->uploadFileSizeLimit) {
                        printf("[%s] %s (%s) -> [%s]\r\n", $ticketId, $filePath, sizeConvert($fileSize), colorize('File Too Large', 'WARN'));
                    } else {
                        $supportUtils = $this->model('SupportUtilsModel');
                        $supportUtils->qcloudModel = $this->model('QCloudModel');
                        $supportUtils->ticketId = $ticketId;
                        $supportUtils->filePath = $filePath;
                        $uploadResult = $supportUtils->uploadFile();
                        if ($uploadResult->result === "Success") {
                            printf("[%s] %s (%s) -> [%s]\r\n", $ticketId, $filePath, sizeConvert($fileSize), colorize('OK', 'SUCCESS'));
                            ++$fileCount;
                        } else {
                            if (isset($uploadResult->status) && $uploadResult->status == 3000) {
                                printf("[%s] %s -> [%s]\r\n", $ticketId, $filePath, colorize('No QID access token', 'ERROR'));
                            } else if(isset($uploadResult->result->errCode) && $uploadResult->result->errCode == 28){
                                printf("[%s] %s -> [%s]\r\n", $ticketId, $filePath, colorize('Request Timeout', 'ERROR'));
                            } else {
                                printf("[%s] %s -> [%s]\r\n", $ticketId, $filePath, colorize($uploadResult->httpStatusCode, 'ERROR'));
                            }
                        }
                    }
                }
            }
            printf("\r\n[%s] %s files has been uploaded.\r\n", $ticketId, colorize($fileCount, 'SUCCESS'));
        } else {
            printf("\r\n[%s] Remote session is not enabled.\r\n", colorize('ERROR', 'ERROR'));
        }
    }
    
    public function registerExternalLog($appName, $appLogPath)
    {
        //php_sapi_name()

        if(filter_var($appLogPath, FILTER_VALIDATE_URL) && "cli" === strtolower(php_sapi_name())){ 

            $supportUtils = $this->model('SupportUtilsModel');
            if (file_exists($appLogPath) && is_dir($appLogPath)) {
                printf("\r\n[%s] You should assign a log file, not folder.\r\n", colorize($appName, 'ERROR'));
            } else if (file_exists($appLogPath) && !is_dir($appLogPath)) {
                if ($supportUtils->setExternalLog($appName, $appLogPath)) {
                    printf("\r\n[%s] Log path %s was registered.\r\n", colorize($appName, 'SUCCESS'), colorize($appLogPath, 'SUCCESS'));
                } else {
                    printf("\r\n[%s] Register external log failed.\r\n", colorize($appName, 'ERROR'), colorize($appLogPath, 'ERROR'));
                }
            } else {
                printf("\r\n[%s] Log file not found.\r\n", colorize($appName, 'ERROR'));
            }   
        }else{
            printf("\r\n[%s] Log file not found.\r\n", colorize($appName, 'ERROR'));
        }
        
        
    }
    public function unRegisterExternalLog($appName)
    {
        if("cli" === strtolower(php_sapi_name())){
            $supportUtils = $this->model('SupportUtilsModel');
            if ($supportUtils->removeExternalLog($appName)) {
                printf("\r\n[%s] External log was unregistered.\r\n", colorize($appName, 'SUCCESS'));
            } else {
                printf("\r\n[%s] Unregister external log failed.\r\n", colorize($appName, 'ERROR'));
            }   
        } else {
                printf("\r\n[%s] Unregister external log failed.\r\n", colorize($appName, 'ERROR'));
        } 
        
    }
    public function listRegisteredExternalLog()
    {
        if("cli" === strtolower(php_sapi_name())){
            $supportUtils = $this->model('SupportUtilsModel');
            if ($rcExternalLogs = $supportUtils->getExternalLog()) {
                $mask = "| %-10.10s | %-50.50s | %-20.20s |\n";
                printf("\r\n" . $mask, 'App Name', 'Log Path', 'Create Time');
                foreach ($rcExternalLogs as $v) {
                    printf($mask, $v['appName'], $v['appLogPath'], date("Y-m-d H:i:s", $v['createdTime']));
                }
                echo "\r\n";
            }   
        }
        
    }
}
