<?php

namespace App\Controllers\Cli;

class WatchDog extends \LMVC\System\Controller
{
    protected $remoteAccessModel;
    protected $logModel;
    protected $tmpKeyPath;
    protected $forwardSrvPort = 22;

    public function __construct()
    {
        parent::__construct();

        $this->remoteAccessModel = $this->model('RemoteAccessModel');
        $this->qtsModel = $this->model('QTSModel');
        $this->logModel = $this->model('LogModel');
        $this->settingsModel = $this->model('SettingsModel');
        $this->fileLogModel = $this->model('FileLogModel');
        $this->remoteAccessModel->fileLogModel = $this->fileLogModel;
        $this->qcloudModel = $this->model('QCloudModel');
        $this->qcloudModel->fileLogModel = $this->fileLogModel;
        $this->remoteAccessModel->qcloudModel = $this->qcloudModel;

        $this->tmpKeyPath = config('app_dir') . '/data/helpdesk_temp_key';

        // if (strtolower(php_sapi_name()) !== 'cli') {
        // $this->fileLogModel->logError('You can not use this function via web.', __FILE__);
        // die('You can not use this function via web. File: ' . __FILE__);
        // }
    }

    public function test($a, $b)
    {
        $data = array('Spark', $a, $b);
        $json = json_encode($data);
        echo $json;
    }
    public function uninstall()
    {
        $status = $this->remoteAccessModel->getStatus();
        $this->remoteAccessModel->ticketId = $status['ticketId'];
        $this->remoteAccessModel->email = $status['email'];

        $ticketId = $this->remoteAccessModel->ticketId;
        $email = $this->remoteAccessModel->email;
        $tempAccount = $this->remoteAccessModel->getTempAccount();
        $supportID = $tempAccount['tempId'];

        if ($ticketId === null) {
            $this->fileLogModel->logError('Missing parameter: ticketId', __FILE__, __LINE__);
            return;
            //Response::error(400, 'Missing parameter: ticketId');
        }

        if ($email === null) {
            $this->fileLogModel->logError('Missing parameter: email', __FILE__, __LINE__);
            return;
            //Response::error(400, 'Missing parameter: email');
        }

        $this->remoteAccessModel->ticketId = $ticketId;
        $this->remoteAccessModel->email = $email;

        $remoteStatus = $this->remoteAccessModel->getRemoteSessionStatus();

        if (is_array($remoteStatus)) {
            $objRStatus = json_encode($remoteStatus);
            $objRStatus = json_decode($objRStatus);

            if (isset($objRStatus->status)) {
                //Response::error($objRStatus->status, apiFormater($objRStatus->msg, 2014));
                $this->fileLogModel->logError($objRStatus->status." : ".apiFormater($objRStatus->msg, 2014), __FILE__, __LINE__);
                return;
            }
        }

        if ($this->remoteAccessModel->isExist($ticketId, $email) && ($remoteStatus == 1000 || $remoteStatus == 1003 || $remoteStatus == 1)) {
            $this->fileLogModel->logDebug('Remote access case is already disabled.', __FILE__, __LINE__);
            //Response::error(403, apiFormater(null, 2008));
            return;
        }

        if ($this->remoteAccessModel->isExist($ticketId, $email) && $remoteStatus == 1004) {
            $this->fileLogModel->logDebug('Remote access case is already expired.', __FILE__, __LINE__);
            //Response::error(403, apiFormater(null, 2018));
            return;
        }

        if ($this->remoteAccessModel->isExist($ticketId, $email) && $remoteStatus == 1005) {
            $this->remoteAccessModel->ticketId = $ticketId;
            $remoteInfo = $this->remoteAccessModel->getRemoteSessionInfo();
            $finishedTime = strtotime($remoteInfo->data->finished_time);
            $this->fileLogModel->logDebug('Remote access case is already closed.', __FILE__, __LINE__);
            //Response::error(403, apiFormater(array('finished_time' => date('Y/m/d H:i:s', $finishedTime)), 2009));
            return;
        }

        $forwardPorts = $this->remoteAccessModel->getForwardPort($ticketId);

        $forwardPort = $forwardPorts[0];
        $sshFwPort = $forwardPort['sshForwardingPort'];
        $httpFwPort = $forwardPort['qtsForwardingPort'];
        $httpsFwPort = $forwardPort['httpsForwardingPort'];
        $forwardIP = $forwardPort['forwardingServer'];
        
        // Kill Reverse SSH tunnel
        $this->remoteAccessModel->sshFwPort = $sshFwPort;
        $this->remoteAccessModel->httpFwPort = $httpFwPort;
        $this->remoteAccessModel->httpsFwPort = $httpsFwPort;
        $this->remoteAccessModel->killSSHTunnel();

        // Remove SSH publickey from forwarding server
        $this->remoteAccessModel->removeFwSSHPKey();

        // Restore .ssh/authorized_keys
        $this->remoteAccessModel->resetAuthedKey();
        $this->remoteAccessModel->restoreAuthedKey();

        // Remove support account - cAN'T BE DONE AS $SERVER component association
       //  $this->remoteAccessModel->removeQTSUserByAPI();
        if ($this->remoteAccessModel->checkQTSUser()) {
            $this->remoteAccessModel->removeQTSUser();
        }

        // Tell Server this ticket is close
        $setRemoteDisable = $this->remoteAccessModel->setRemoteSessionDisable();
        if ($setRemoteDisable !== true) {
            $this->remoteAccessModel->ticketId = $ticketId;
            $remoteInfo = $this->remoteAccessModel->getRemoteSessionInfo();
            $this->fileLogModel->logError('Can not disable remote session.', __FILE__, __LINE__);
            //Response::error(500, apiFormater(array('finished_time' => date('Y/m/d H:i:s', $finishedTime)), 2009));
            return;
        }

        //Close SSH if it Enable by Helpdesk
        if($this->settingsModel->getNeedDisableSSH()==1){
            $this->qtsModel->sshDisable();
            $this->settingsModel->setNeedDisableSSH(0);
        }

        // Set local db
        $this->remoteAccessModel->setDisable();

        // Kill WatchDog (err ahh!)
        $watchDog = new \App\Controllers\Cli\WatchDog();
        $bark = $watchDog->liveOrDie();

        // Disable work screen
        $this->remoteAccessModel->setWorkScreen(0);


        echo "Disabled Helpdesk";

    }
    public function liveOrDie()
    {
        $status = $this->remoteAccessModel->getActivatedStatus();
        if (empty($status)) {
            //status is disabled
            $this->fileLogModel->logDebug('Exit watchdog');
            shell_exec('qsh helpdeskd.disableWatchdog');

            return 'Disable WatchDog';
        } else {
            $this->fileLogModel->logDebug('Watchdog woof');
            shell_exec('qsh helpdeskd.enableWatchdog');

            return 'Enable WatchDog';
        }
    }

    public function woof()
    {
        $this->qcloudModel->refreshToken();

        // Fix SSHD config permission
        $this->qtsModel->fixSSHDConfigPerm();

        //從DB抓取目前的status
        $status = $this->remoteAccessModel->getStatus();

        $forwardHttpPort = $status['qtsForwardingPort'];
        $forwardHttpsPort = $status['httpsForwardingPort'];
        $forwardSSHPort = $status['sshForwardingPort'];
        $forwardIP = $status['forwardingServer'];

        $this->remoteAccessModel->sshFwPort = $forwardSSHPort;
        $this->remoteAccessModel->httpFwPort = $forwardHttpPort;
        $this->remoteAccessModel->httpsFwPort = $forwardHttpsPort;

        // Get session status from support center
        $this->remoteAccessModel->ticketId = $status['ticketId'];
        $this->remoteAccessModel->email = $status['email'];
        $this->fileLogModel->logDebug("current ".$status['ticketId']);
        $remoteStatus = $this->remoteAccessModel->getRemoteSessionStatus();

        if (isset($remoteStatus) && $remoteStatus == 'FALSE') {
            $this->fileLogModel->logError('Can not get session status from support center.', __FILE__, __LINE__);
            die();
        }

        if (empty($status)) {
            $remoteStatus = 1003;
        }

        switch ($remoteStatus) {
            case '1001': // Activated
            case '1002': // In Progress
                $hasSshTunnel = false;
                $hasQtsTunnel = false;
                $hasHttpsTunnel = false;

                //檢查是否過期
                $expireTime = new \DateTime($status['expireTime']);
                $now = new \DateTime('now');
                if ($now > $expireTime) {
                    //過期就disable
                    // Disable Remote Session
                    $this->remoteAccessModel->disableRemoteSession();

                    // Set local db
                    $this->remoteAccessModel->setExpired();

                    $this->fileLogModel->logDebug('Remote access session expired.');
                    echo "Remote access session expired\n";

                    return;
                }

                $this->fileLogModel->logDebug('Current Status: ' . $remoteStatus);
                echo 'Current Status: ' . $remoteStatus . "\n";

                // Renew SSH key for support to client
                $this->remoteAccessModel->tmpKeyPath = $this->tmpKeyPath;
                $this->remoteAccessModel->ticketId = $status['ticketId'];

                $this->remoteAccessModel->resetAuthedKey();
                $this->remoteAccessModel->restoreAuthedKey();

                $sshKeypair = $this->remoteAccessModel->generateSSHKeypair();
                $this->remoteAccessModel->publicKey = $sshKeypair['PK'];
                $this->remoteAccessModel->base64SK = $sshKeypair['encSK'];
                $resultRenewKey = $this->remoteAccessModel->renewSSHKeypair();

                shell_exec("chown www-data:www-data -R ".$this->tmpKeyPath);

                if (is_array($resultRenewKey)) {
                    $this->fileLogModel->logDebug('SSH keypair renew failed: ' . $resultRenewKey['result']);
                } else {
                    $this->fileLogModel->logDebug('SSH keypair renewed: ' . $status['ticketId']);
                }

                //檢查process裡有沒有tunnel的ssh
                $cmd = 'ps ax| grep "/usr/bin/ssh -NfR" | grep "ubuntu@" | grep -v "grep" | awk \'{print $1,"\t"$3,"\t"$7}\'';

                exec($cmd, $output);

                //重啟需要的資訊
                $httpPort = shell_exec("/usr/bin/qgetcfg --app=SYS_CONF 'system-web-server' 'Web Access Port' -d 80");
                $httpsPort = shell_exec("/usr/bin/qgetcfg --app=SYS_CONF 'system-web-server' 'HTTPS Port' -d 443");
                $sshPort = shell_exec("/usr/bin/qgetcfg --app=SYS_CONF 'Login' 'SSH Port' -d 22");
                $httpPort = str_replace("\n", '', $httpPort);
                $httpsPort = str_replace("\n", '', $httpsPort);
                $sshPort = str_replace("\n", '', $sshPort);

                $this->remoteAccessModel->sshPort = $sshPort;
                $this->remoteAccessModel->httpPort = $httpPort;
                $this->remoteAccessModel->httpsPort = $httpsPort;
                $this->remoteAccessModel->forwardIP = $forwardIP;
                $this->remoteAccessModel->forwardSrvPort = $this->forwardSrvPort;
                $this->remoteAccessModel->fwSKeyPath = config('app_dir') . '/data/forwarding.pem';
                shell_exec("chown www-data:www-data -R ".config('app_dir') . '/data/forwarding.pem');

                foreach ($output as $line) {
                    $line = trim($line);

                    $psInfo = explode("\t", $line);
                    //psInfo 0: pid, 1: process status, 2: forwarding info

                    $forwardInfo = explode(':', $psInfo[2]);
                    $forwardToPort = $forwardInfo[0];

                    if ($forwardToPort == $forwardHttpPort) {
                        if (strtolower($psInfo[1]) != 'z') {
                            $hasQtsTunnel = true;
                        } else {
                            //變成殭屍就幹掉
                            exec("kill {$psInfo[0]}");
                            $this->fileLogModel->logDebug("Kill dead QTS tunnel process {$psInfo[0]}.");
                            echo "Kill dead QTS tunnel process {$psInfo[0]}\n";
                        }
                    }

                    if ($forwardToPort == $forwardHttpsPort) {
                        if (strtolower($psInfo[1]) != 'z') {
                            $hasHttpsTunnel = true;
                        } else {
                            //變成殭屍就幹掉
                            exec("kill {$psInfo[0]}");
                            $this->fileLogModel->logDebug("Kill dead QTS secured tunnel process {$psInfo[0]}.");
                            echo "Kill dead QTS secured tunnel process {$psInfo[0]}\n";
                        }
                    }

                    if ($forwardToPort == $forwardSSHPort) {
                        if (strtolower($psInfo[1]) != 'z') {
                            $hasSshTunnel = true;
                        } else {
                            //變成殭屍就幹掉
                            exec("kill {$psInfo[0]}");
                            $this->fileLogModel->logDebug("Kill dead SSH tunnel process {$psInfo[0]}.");
                            echo "Kill dead SSH tunnel process {$psInfo[0]}\n";
                        }
                    }
                }

                if (!$hasQtsTunnel) {
                    //qts tunnel不存在要重啟
                    $this->remoteAccessModel->establishHTTPFwTunnel();
                    $this->fileLogModel->logDebug('Restart QTS tunnel.');
                    echo "Restart QTS tunnel\n";

                    sleep(1);
                    $pidArray = $this->remoteAccessModel->getSSHTunnelPid();

                    if ($pidArray !== false && empty($pidArray['httpFwPid'])) {
                        // Remove SSH publickey from forwarding server
                        $this->remoteAccessModel->removeFwSSHPKey();

                        // Change SSH Keypair for forwarding server
                        $this->remoteAccessModel->generateFwKeypair();

                        // Try to reestablish connection
                        $this->remoteAccessModel->establishHTTPFwTunnel();
                        $this->fileLogModel->logDebug('Restart QTS tunnel again.');
                        echo "Restart QTS tunnel again\n";
                    }
                }

                if (!$hasHttpsTunnel) {
                    //QTS HTTPS tunnel不存在要重啟
                    $this->remoteAccessModel->establishHTTPSFwTunnel();
                    $this->fileLogModel->logDebug('Restart QTS HTTPS tunnel.');
                    echo "Restart QTS HTTPS tunnel\n";

                    sleep(1);
                    $pidArray = $this->remoteAccessModel->getSSHTunnelPid();

                    if ($pidArray !== false && empty($pidArray['httpsFwPid'])) {
                        // Remove SSH publickey from forwarding server
                        $this->remoteAccessModel->removeFwSSHPKey();

                        // Change SSH Keypair for forwarding server
                        $this->remoteAccessModel->generateFwKeypair();

                        // Try to reestablish connection
                        $this->remoteAccessModel->establishHTTPSFwTunnel();
                        $this->fileLogModel->logDebug('Restart QTS HTTPS tunnel again.');
                        echo "Restart QTS HTTPS tunnel again\n";
                    }
                }

                if (!$hasSshTunnel) {
                    //ssh tunnel不存在要重啟
                    $this->remoteAccessModel->establishSSHFwTunnel();
                    $this->fileLogModel->logDebug('Restart SSH tunnel.');
                    echo "Restart SSH tunnel\n";

                    sleep(1);
                    $pidArray = $this->remoteAccessModel->getSSHTunnelPid();

                    if ($pidArray !== false && empty($pidArray['sshFwPid'])) {
                        // Remove SSH publickey from forwarding server
                        $this->remoteAccessModel->removeFwSSHPKey();

                        // Change SSH Keypair for forwarding server
                        $this->remoteAccessModel->generateFwKeypair();

                        // Try to reestablish connection
                        $this->remoteAccessModel->establishSSHFwTunnel();
                        $this->fileLogModel->logDebug('Restart SSH tunnel again.');
                        echo "Restart SSH tunnel again\n";
                    }
                }
                $this->remoteAccessModel->setStatus($remoteStatus, $status['ticketId'], $status['email']);
                $logInProgressIsSet = $this->logModel->isRemoteLogSet('notification_remoteSupport_inProgress');

                if ($remoteStatus == 1002 && !$logInProgressIsSet) {
                    $this->logModel->qtsLog(3);
                    //$this->logModel->qtsLogInfo("Your remote support session is starting, please do not change the NAS settings during this period.");
                    $this->logModel->setRemoteLog('notification_remoteSupport_inProgress');
                    $this->logModel->setAppLog('notification_remoteSupport_inProgress');
                }

                break;

            case '1005': // Closed
                // Disable Remote Session
                $this->remoteAccessModel->disableRemoteSession();

                // Set local db
                $this->remoteAccessModel->setClosedInfo();

                $this->fileLogModel->logDebug('Status is closed, exit watchdog.');
                echo "Status is closed, exit watchdog\n";

                //Close SSH if it Enable by Helpdesk
                if($this->settingsModel->getNeedDisableSSH()==1){
                    $this->qtsModel->sshDisable();
                    $this->settingsModel->setNeedDisableSSH(0);
                }   

                $logClosedIsSet = $this->logModel->isRemoteLogSet('notification_remoteSupport_finished');

                if (!$logClosedIsSet) {
                    $this->logModel->qtsLog(4);
                    //$this->logModel->qtsLogInfo("Your remote support session was finished, please check for result.");
                    $this->logModel->setRemoteLog('notification_remoteSupport_finished');
                    $this->logModel->setAppLog('notification_remoteSupport_finished');
                }

                break;

            case '1000': // Wait for Activate
                // Disable Remote Session
                $this->remoteAccessModel->disableRemoteSession();

                // Set local db
                $this->remoteAccessModel->setWaitActivate();

                $this->fileLogModel->logDebug('Status is waitting for activate, exit watchdog.');
                echo "Status is waitting for activate, exit watchdog\n";

                break;

            case '1003': // Disabled
                // Disable Remote Session
                $this->remoteAccessModel->disableRemoteSession();

                // Set local db
                $this->remoteAccessModel->setDisable();

                //Close SSH if it Enable by Helpdesk
                if($this->settingsModel->getNeedDisableSSH()==1){
                    $this->qtsModel->sshDisable();
                    $this->settingsModel->setNeedDisableSSH(0);
                }

                $this->fileLogModel->logDebug('Status is disabled, exit watchdog.');
                echo "Status is disabled, exit watchdog\n";

                break;

            case '1004': // Expired
                // Disable Remote Session
                $this->remoteAccessModel->disableRemoteSession();

                // Set local db
                $this->remoteAccessModel->setExpired();

                //Close SSH if it Enable by Helpdesk
                if($this->settingsModel->getNeedDisableSSH()==1){
                    $this->qtsModel->sshDisable();
                    $this->settingsModel->setNeedDisableSSH(0);
                }

                $this->fileLogModel->logDebug('Status is expired, exit watchdog.');
                echo "Status is expired, exit watchdog\n";

                break;
        }

        $this->liveOrDie();
    }
}
