<?php

namespace App\Models;

use RecursiveIteratorIterator;
use RecursiveDirectoryIterator;
use \DateTime;

libxml_use_internal_errors(true);

class CustomPortalModel extends \LMVC\System\Model
{
    protected $apiUrl = '/api/v2';
    protected $langMap = [
        'ENG'=>'en',
        'SCH'=>'zh-cn',
        'TCH'=>'zh-tw',
        'CZE'=>'cs-cz',
        'DAN'=>'en',
        'GER'=>'de-de',
        'SPA'=>'es-es',
        'FRE'=>'fr-fr',
        'ITA'=>'it-it',
        'JPN'=>'ja-jp',
        'KOR'=>'ko-kr',
        'NOR'=>'en',
        'POL'=>'pl-pl',
        'RUS'=>'ru-ru',
        'FIN'=>'en',
        'SWE'=>'sv-se',
        'DUT'=>'nl-nl',
        'TUR'=>'tr-tr',
        'THA'=>'th-th',
        'POR'=>'pt-pt',
        'HUN'=>'hu-hu',
        'GRK'=>'en',
        'ROM'=>'en',
    ];

    public function __construct()
    {
        parent::__construct();
        $this->lang='ENG';
        $this->apiGWUrl = "https://".config('api_gateway_server');
        $this->cpUrl = 'https://'.config('customPortal_server');
    }
    public function customPortalServer(){
        return 'https://'.config('customPortal_server');
    }

    public function generateSystemLog()
    {
        $sysInfo = $this->getSystemInfo();
        $logFilename = $sysInfo['hwSN'];
        $logPath = "/tmp/helpdump/{$logFilename}.zip";
        
        shell_exec("qsh helpdeskd.gen_log  -v '{\"name\":\"".$logFilename."\"}'");

        if (file_exists($logPath)) {
            return $logPath;
        } else {
            return;
        }
    }

    public function getSystemInfo()
    {
        $cmd="qsh system.'GET /system/firmware/info'"; 
        $getSysInfo = shell_exec($cmd);
        $getSysInfo = json_decode($getSysInfo);

        if (!empty($getSysInfo->firmware->name)){
            $hwModel = (string) $getSysInfo->firmware->display_name;
        } else {
            $hwModel = str_replace("\n", '', shell_exec('/usr/bin/qgetcfg --app=MODEL "System Enclosure" Model'));
        }

        $hwVer = $getSysInfo->firmware->version;
        $cmd="qsh system.'GET /system/info' '{\"__HEADER__\":{\"QUERY_STRING\":\"subfunc=sysinfo&hd=no\"}}'";


        $hwSN = shell_exec($cmd);
        $hwSN = simplexml_load_string($hwSN, 'SimpleXMLElement', LIBXML_NOERROR | LIBXML_NOCDATA | LIBXML_NOBLANKS);
        $hwSN=json_encode($hwSN);
        $hwSN=json_decode($hwSN);
        if(!empty($hwSN->func->ownContent->root->serial_number))
            $hwSN = $hwSN->func->ownContent->root->serial_number;
        else
            $hwSN = "unknown";

        return array(
            'hwModel' => $hwModel,
            'hwVer' => $hwVer,
            'hwSN' => $hwSN,
        );
    }

    

    public function getFirmwareVersion($lang = 'en')
    {
        return $this->getCustomPortalData("firmware-version");
    }
    public function getCountry()
    {
        return $this->getCustomPortalData("country");
    }

    public function getIssueCategory()
    {
        return $this->getCustomPortalData("issue-category-qpkg");
    }

    public function getOs()
    {
        return $this->getCustomPortalData("operating-system"); 
    }

    public function getCustomer()
    {
        return $this->getCustomPortalData("typeof-customer"); 
    }

        public function getTimezone()
    {
        return $this->getCustomPortalData("timezone"); 
    }


    public function getCustomPortalData($api){
        $mlang = 'en';
        if(isset($this->langMap[$this->lang]))$mlang = $this->langMap[$this->lang];
        $curlUrl =  $this->customPortalServer()."{$this->apiUrl}/data/{$api}/{$mlang}";
        $srvOutput = curlWrapper('GET', $curlUrl);
        if (is_array($srvOutput)){
            $srvOutput = json_encode($srvOutput);
            $srvOutput = json_decode($srvOutput);

            if (isset($srvOutput->status) && $srvOutput->status === 'FALSE') {
                if (isset($srvOutput->result->errCode) && $srvOutput->result->errCode == 28) {
                    return array('status' => '408', 'msg' => $srvOutput->result->errMsg);
                } else {
                    return false;
                }
            }
        }

        $srvOutput=json_decode($srvOutput);
        if(isset($srvOutput->result) 
            && strtoupper($srvOutput->result) == "SUCCESS" 
            && isset($srvOutput->response) 
            && count($srvOutput->response) > 0
            && isset($srvOutput->response[0]->API_KEY)
            && !empty($srvOutput->response[0]->API_KEY)
        ){
            $arr=$srvOutput->response;
            usort($arr, function($a, $b) {
                return strcmp($a->API_KEY,$b->API_KEY);
            });
            return $arr;
        }else 
            if(isset($srvOutput->response[0]->sf_key) && !empty($srvOutput->response[0]->sf_key)){
                //timezone
                return $srvOutput->response;
            }else{
                return false;
            }
            

        
    }

    public function getUpdateWarn()
    {

        $curlUrl = 'https://'.config('customPortal_server')."{$this->apiUrl}/data/verifyversion?internalName=helpdesk&version='.APP_VERSION.'&os_type=QTS";
        $srvOutput = curlWrapper('GET', $curlUrl);

        return $srvOutput;
    }

    

    public function createTicket()
    {
        $curlUrl = $this->apiGWUrl ."/service/api/v2/case/case";
        $token = $this->qcloudModel->getCachedToken();
        if(!$token['access_token']){
            return array('status' => 3000, 'result' => null);
        }  

        $postData = array(
            "productSerialNumber"=> $this->hwSN,
            "productModel"=> $this->model,
            "firmwareVersion"=> $this->hwVer,
            "clientOperatingSystem"=> $this->osType,
            "issueCategoryType"=> $this->issueCategory1,
            "issueCategory"=> $this->issueCategory2,
            "subIssueCategory"=> $this->issueCategory3,
            "caseSubject"=> $this->caseSubject,
            "description"=> $this->description,
        );
        
        $backendData = array(
            "firstName" => $this->firstName,
            "lastName"=> $this->lastName,
            "email"=> $this->email,
            "country"=> $this->country,
            "typeOfCustomer"=> $this->customer,
            "telephone"=> $this->phone,
            "companyName"=> $this->company,
            "timeZone"=> $this->timezone->sf_key,
            "qnap_user_id" => $this->qnap_user_id,
            "qnap_email"=> $this->qnap_email,
            "qnap_phone"=> $this->qnap_phone,
        );

        if(intval($this->osType) === -1)unset($postData['clientOperatingSystem']);
        if(intval($this->customer) === -1)unset($backendData['typeOfCustomer']);

        $curlPostFields = http_build_query(array_merge($postData,$backendData));

        $header =array(
            'X-Qid-Authorization: '.$token['access_token'],
        );

        
        $srvOutput = curlWrapper('POST', $curlUrl, $curlPostFields,false,$header);
        
        if (is_array($srvOutput)) {
            $srvOutput = json_encode($srvOutput);
            $srvOutput = json_decode($srvOutput);

            if (isset($srvOutput->status) && $srvOutput->status === 'FALSE') {
                if (isset($srvOutput->result->errCode) && $srvOutput->result->errCode == 28) {
                    return array('status' => '408', 'msg' => $srvOutput);
                } else {
                    return array('status' => '408', 'msg' => $srvOutput);
                }
            }
        }

        $srvOutput = json_decode($srvOutput);

        if ($srvOutput === false) {
            return array('status' => '408', 'msg' => 'post failed');
        }

        if (empty($srvOutput->response->caseId)) {
            return array('status' => '408', 'msg' => 'No CastId','response'=>$srvOutput);
        }
        
        return array(
            'result' => 'Success',
            'caseId' => (string) $srvOutput->response->caseId,
            'caseNo' => (string) $srvOutput->response->caseNo,
            'contactId' => (string) $srvOutput->response->contactId,
            'replyRecordId'=> (string) $srvOutput->response->replyRecordId,
        );
         
    }

    /* {{{ public function uploadTicketAttachment() { */
    public function uploadTicketAttachment()
    {
        $curlUrl = $this->cpUrl ."/api/v2/casefile/upload";
        $token = $this->qcloudModel->getCachedToken();
        if(!$token['access_token']){
            return array('status' => 3000, 'result' => null);
        } 
        $postData = array(
            'caseId' => $this->caseId,
            'replyRecordId' => $this->replyRecordId,
            'mode' => 'ticket',
            'file' => $this->file
        );
        $header =array(
            'X-Qid-Authorization: '.$token['access_token'],
        );

        //$curlPostFields = http_build_query($postData);

        $srvOutput = curlWrapper('POST', $curlUrl, $postData,false,$header);

        if (is_array($srvOutput)) {
            $srvOutput = json_encode($srvOutput);
            $srvOutput = json_decode($srvOutput);

            if (isset($srvOutput->status) && $srvOutput->status === 'FALSE') {
                if (isset($srvOutput->result->errCode) && $srvOutput->result->errCode == 28) {
                    return array('status' => '408', 'msg' => $srvOutput->result->errMsg);
                } else {
                    return false;
                }
            }
        }

        $srvOutput = json_decode($srvOutput);

        if ($srvOutput->result != 'Success') {
            return false;
        }
        return true;
    }

    public function getFirmwareVersionID()
    {
        $productName = $this->FirmwareVersion;

        $customFieldOption = $this->getFirmwareVersion();

        $arrResult = array();
        $result = '';

        if (!$customFieldOption || isset($customFieldOption->status)) {
            return false;
        }

        //$objProduct = get_object_vars($customFieldOption);
        $pattern = "/\.*{$productName}\.*/i";

        foreach ($customFieldOption as $key => $val) {
            if (preg_match($pattern, $val->VALUE)) {
                $arrResult[$val->VALUE] = $val->API_KEY;
                $result = $val->API_KEY;
            }
        }

        if (empty($arrResult)) {
            return false;
        }

        if (count($arrResult) > 1) {
            foreach ($arrResult as $key => $val) {
                if ($productName == $val->VALUE) {
                    return $val;
                    break;
                }
            }
        } elseif (isset($arrResult[$result])) {
            return $arrResult[$result];
        } else {
            return false;
        }
    }

    public function getExternalLog()
    {
        $queryStr = "SELECT * FROM external_log";
        $rcExternalLogs = null;
        $arrExternalLog = array();

        try {
            $rcExternalLogs = $this->db->query($queryStr);
        } catch (\Exception $e) {
            return false;
        }

        foreach ($rcExternalLogs as $v) {
            if (file_exists($v['appLogPath']) && !is_dir($v['appLogPath'])) {
                array_push($arrExternalLog, $v['appLogPath']);
            }
        }

        return $arrExternalLog;
    }





}
