<?php

namespace App\Models;

class DbUtilsModel extends \LMVC\System\Model
{
    public function __construct()
    {
        parent::__construct();
    }

    public function initDB()
    {
        parent::__construct();
    }

    public function getSchemaVersion()
    {
        $schemaVersion = '';

        try {
            $schemaVersion = $this->db->querySingleRow("SELECT cfgValue FROM configuration WHERE cfgKey='schemaVersion'");
        } catch (\Exception $e) {
            return false;
        }

        return $schemaVersion;
    }

    public function isValidDB()
    {
        try {
            return $this->db->queryDataIsExist("SELECT cfgValue FROM configuration WHERE cfgKey='showGuides'");
        } catch (\Exception $e) {
            return false;
        }
    }

    public function migrateInitDB()
    {
        $schemaVersion = 1;
        $rowCount = 0;

        $sqlInitConfiguration = array(
            "DROP TABLE IF EXISTS 'configuration'",
            "CREATE TABLE configuration (cfgKey TEXT NOT NULL, cfgValue TEXT, PRIMARY KEY (cfgKey))",
            "INSERT INTO 'configuration' VALUES ('tempId', 'www-data')",
            "INSERT INTO 'configuration' VALUES ('showGuides', 1)",
            "INSERT INTO 'configuration' VALUES ('tempPw', null)",
            "INSERT INTO 'configuration' VALUES ('msgRetainDays', 90)",
            "INSERT INTO 'configuration' VALUES ('locRegion', -1)",
            "INSERT INTO 'configuration' VALUES ('locArea', -1)",
            "INSERT INTO 'configuration' VALUES ('attachSyslog', 0)",
            "INSERT INTO 'configuration' VALUES ('showDiagWarning', 1)",
            "INSERT INTO 'configuration' VALUES ('showDiagStressWarning', 1)",
            "INSERT INTO 'configuration' VALUES ('updateWarn', 0)",
            "INSERT INTO 'configuration' VALUES ('qidToken', 0)",
            "INSERT INTO 'configuration' VALUES ('qidAccessToken', 0)",
            "INSERT INTO 'configuration' VALUES ('qidRefreshToken', 0)",

            "INSERT INTO 'configuration' VALUES ('showSshEnableWarning', 1)",
            "INSERT INTO 'configuration' VALUES ('needDisableSSH', 0)",
        );

        $sqlInitRemoteAccessStatus = array(
            "DROP TABLE IF EXISTS 'remoteAccessStatus'",
            "CREATE TABLE remoteAccessStatus (
                cid integer PRIMARY KEY,
                status varchar,
                ticketId text,
                email text,
                startTime text,
                expireTime integer,
                forwardingServer text,
                sshForwardingPort integer,
                qtsForwardingPort integer,
                httpsForwardingPort integer,
                confirmedStatus integer,
                closedTime integer,
                closedMsg text,
                extended integer DEFAULT(0)
            )"
        );

        try {
            foreach ($sqlInitConfiguration as $queryStr) {
                $rowCount += $this->db->queryNoneResult($queryStr);
            }
            foreach ($sqlInitRemoteAccessStatus as $queryStr) {
                $rowCount += $this->db->queryNoneResult($queryStr);
            }
        } catch (\Exception $e) {
            return false;
        }

        return $rowCount;
    }

    public function migrateVer1()
    {
        $schemaVersion = 1;
        $rowCount = 0;

        $sqlMigrateConfiguration = "INSERT INTO 'configuration' VALUES ('schemaVersion', {$schemaVersion})";

        try {
            $rowCount += $this->db->queryNoneResult($sqlMigrateConfiguration);
        } catch (\Exception $e) {
            return false;
        }

        return $rowCount;
    }

    public function migrateVer2()
    {
        $schemaVersion = 2;
        $rowCount = 0;

        $sqlMigrateVer2 = array(
            "CREATE TABLE log (
                id integer PRIMARY KEY,
                msgId text,
                msgMeta text,
                createdTime text
            );",
            "INSERT INTO configuration (cfgKey, cfgValue) VALUES ('lastSetLogMsgId', '');",
            "UPDATE configuration SET cfgValue = {$schemaVersion} WHERE cfgKey = 'schemaVersion';"
        );

        try {
            foreach ($sqlMigrateVer2 as $queryStr) {
                $rowCount += $this->db->queryNoneResult($queryStr);
            }
        } catch (\Exception $e) {
            return false;
        }

        return $rowCount;
    }

    public function migrateVer3()
    {
        $schemaVersion = 3;
        $rowCount = 0;

        $sqlMigrateVer3 = array(
            "CREATE TABLE external_log (
                id integer PRIMARY KEY,
                appName text,
                appLogPath text,
                createdTime text
            );",
            "UPDATE configuration SET cfgValue = {$schemaVersion} WHERE cfgKey = 'schemaVersion';"
        );

        try {
            foreach ($sqlMigrateVer3 as $queryStr) {
                $rowCount += $this->db->queryNoneResult($queryStr);
            }
        } catch (\Exception $e) {
            return false;
        }

        return $rowCount;
    }
}
