<?php

namespace App\Models;

class FileLogModel extends \LMVC\System\Toolbox
{
    public function __construct()
    {
        $logList = array();

        $date = date('Y-m-d');
        $fileName = "log_{$date}.log";
        $logPath = config('log_path');
        $logKeepDays = config('log_keepdays');

        if (empty($logPath)) {
            $logPath = __DIR__ . '../data/log';
        }
        $logFile = $logPath . '/' . $fileName;

        if (file_exists($logPath) && is_dir($logPath)) {
            if ($handle = opendir($logPath)) {
                while (false !== ($entry = readdir($handle))) {
                    if ($entry != '.' && $entry != '..') {
                        array_push($logList, $logPath . '/' . $entry);
                    }
                }
                closedir($handle);
            }
        }

        if (!in_array($logFile, $logList)) {
            array_push($logList, $logFile);
        }

        sort($logList);

        for ($i = 0; $i < count($logList) - $logKeepDays; ++$i) {
            if (file_exists(realpath($logList[$i]))) {
                unlink(realpath($logList[$i]));
                $this->logDebug('Remove log file: ' . $logList[$i]);
            }
        }
    }

    public function logError($msg, $file = '', $line = '')
    {
        if (empty($file)) {
            $this->log('error', $msg);
        } else {
            $this->log('error', '[' . $file . '] ' . $line . ': ' . $msg);
        }
    }

    public function logDebug($msg, $file = '', $line = '')
    {
        if (empty($file)) {
            $this->log('debug', $msg);
        } else {
            $this->log('debug', '[' . $file . '] ' . $line . ': ' . $msg);
        }
    }
}
