<?php

namespace App\Models;

use RecursiveIteratorIterator;
use RecursiveDirectoryIterator;

libxml_use_internal_errors(true);

class QCloudModel extends \LMVC\System\Model
{
    protected $loginUrl;
    protected $authUrl;
    protected $client_id;
    protected $client_secret;
    
    public function __construct()
    {
        parent::__construct();
        $this->apiUrl = "https://".config('api_gateway_server');
    }


    public function login($username="",$pwd="",$curuser="")
    {
         
        $curlUrl = $this->apiUrl."/oauth2/token";
        $postData = array(
            "grant_type"=>"password",
            "username"=>$username,
            "password"=>$pwd
        );

        $curlPostFields = json_encode($postData);
        $srvOutput = curlWrapper('POST', $curlUrl, $curlPostFields, null, ['Content-Type: application/json']);
        if (is_array($srvOutput)) {
            $srvOutput = json_encode($srvOutput);
            $srvOutput = json_decode($srvOutput);
            return $srvOutput;
        }else{
            $srvOutput = json_decode($srvOutput);
        }

        if($srvOutput->access_token){
            $expires = time()+$srvOutput->expires_in;
            $this->db->query("INSERT OR REPLACE INTO configuration ('cfgKey','cfgValue') VALUES ('qidCuruser',?)",[$curuser]);
            $this->db->query("INSERT OR REPLACE INTO configuration ('cfgKey','cfgValue') VALUES ('qidAccount',?)",[$username]);
            $this->db->query("INSERT OR REPLACE INTO configuration ('cfgKey','cfgValue') VALUES ('qidAccessToken',?)",[$srvOutput->access_token]);
            $this->db->query("INSERT OR REPLACE INTO configuration ('cfgKey','cfgValue') VALUES ('qidRefreshToken',?)",[$srvOutput->refresh_token]);
            $this->db->query("INSERT OR REPLACE INTO configuration ('cfgKey','cfgValue') VALUES ('qidExpiresIn',?)",[$expires]);
        }
        return  $srvOutput;
    }
    public function bind($username="",$pwd="",$curuser="")
    {
         
        $curlUrl = $this->apiUrl."/oauth2/token";
        $postData = array(
            "grant_type"=>"password",
            "username"=>$username,
            "password"=>$pwd
        );

        $curlPostFields = json_encode($postData);
        $srvOutput = curlWrapper('POST', $curlUrl, $curlPostFields, null, ['Content-Type: application/json']);
        if (is_array($srvOutput)) {
            $srvOutput = json_encode($srvOutput);
            $srvOutput = json_decode($srvOutput);
            return $srvOutput;
        }else{
            $srvOutput = json_decode($srvOutput);
        }

        if($srvOutput->access_token){
            $expires = time()+$srvOutput->expires_in;
            $this->db->query("INSERT OR REPLACE INTO configuration ('cfgKey','cfgValue') VALUES ('qidBindCuruser',?)",[$curuser]);
            $this->db->query("INSERT OR REPLACE INTO configuration ('cfgKey','cfgValue') VALUES ('qidBindAccount',?)",[$username]);
            $this->db->query("INSERT OR REPLACE INTO configuration ('cfgKey','cfgValue') VALUES ('qidBindAccessToken',?)",[$srvOutput->access_token]);
            $this->db->query("INSERT OR REPLACE INTO configuration ('cfgKey','cfgValue') VALUES ('qidBindRefreshToken',?)",[$srvOutput->refresh_token]);
            $this->db->query("INSERT OR REPLACE INTO configuration ('cfgKey','cfgValue') VALUES ('qidBindExpiresIn',?)",[$expires]);
        }
        return  $srvOutput;
    }

    public function logout()
    {   
        $accessToken = $this->db->querySingleRow("SELECT cfgValue FROM configuration WHERE cfgKey='qidAccessToken'");
        $curlUrl = $this->apiUrl."/oauth2/logout";
        $postData = array(
            "access_token"=>$accessToken['cfgValue'],
        );
        $curlPostFields = json_encode($postData);
        $srvOutput = curlWrapper('POST', $curlUrl, $curlPostFields, null, ['Content-Type: application/json']);


        $qidAccount = $this->db->querySingleRow("SELECT cfgValue FROM configuration WHERE cfgKey='qidAccount'");
        $this->db->query("UPDATE configuration SET cfgValue=0 WHERE cfgKey='qidAccessToken'");
        $this->db->query("UPDATE configuration SET cfgValue=0 WHERE cfgKey='qidRefreshToken'");
        $this->db->query("UPDATE configuration SET cfgValue=0 WHERE cfgKey='qidAccount'");

        $obj = new \stdClass;
        $obj->result='SUCCESS';
        $obj->account=$qidAccount['cfgValue'];
        $obj->data = $srvOutput;
        return $obj;
    }

    public function unbind()
    {   
        $accessToken = $this->db->querySingleRow("SELECT cfgValue FROM configuration WHERE cfgKey='qidBindAccessToken'");
        $curlUrl = $this->apiUrl."/oauth2/logout";
        $postData = array(
            "access_token"=>$accessToken['cfgValue'],
        );
        $curlPostFields = json_encode($postData);
        $srvOutput = curlWrapper('POST', $curlUrl, $curlPostFields, null, ['Content-Type: application/json']);


        $qidAccount = $this->db->querySingleRow("SELECT cfgValue FROM configuration WHERE cfgKey='qidBindAccount'");
        $this->db->query("UPDATE configuration SET cfgValue=0 WHERE cfgKey='qidBindAccessToken'");
        $this->db->query("UPDATE configuration SET cfgValue=0 WHERE cfgKey='qidBindRefreshToken'");
        $this->db->query("UPDATE configuration SET cfgValue=0 WHERE cfgKey='qidBindAccount'");

        $obj = new \stdClass;
        $obj->result='SUCCESS';
        $obj->account=$qidAccount['cfgValue'];
        $obj->data = $srvOutput;
        return $obj;
    }

    public function setUserInfo(){
        $token = $this->getCachedToken('qid');
        if(!$token['access_token']){
            return array('status' => 3000, 'result' => null);
        }   

        $curlPostFields = array(
            "first_name"=> $this->firstName,
            "last_name"=> $this->lastName,
            "country"=> $this->country,
        );
        /*
        if(!empty($this->timezone->ac_key)){
            $curlPostFields["timezone"] = [
                "listName"=> $this->timezone->ac_key,
                "listValue"=> $this->timezone->ac_listValue,
                "tzPath"=> $this->timezone->ac_tzPath,
                "offset"=> $this->timezone->ac_offset,
                "tzName"=> $this->timezone->ac_tzName,
            ];
        }*/

        $curlUrl = $this->apiUrl."/oauth2/me";
        $header =array(
            'X-Qid-Authorization: '.$token['access_token'],
            'Content-Type: text/plain'
        );

        $srvOutput = curlWrapper('PUT', $curlUrl, json_encode($curlPostFields),false,$header);
        if (is_array($srvOutput)) {
            $srvOutput = json_encode($srvOutput);
            $srvOutput = json_decode($srvOutput);
        }else{
            $srvOutput = json_decode($srvOutput);
        }

        return $srvOutput;
        
    }


    public function user_info($curuser){
        $token = $this->getCachedToken('qid');
        if(!$token['access_token']){
            return array('status' => 3000, 'result' => null);
        }   
        if($this->isTokenExpires('qid')){
            return array('status' => 3004, 'result' => null);
        }
        $saveCuruser = $this->db->querySingleRow("SELECT cfgValue FROM configuration WHERE cfgKey='qidCuruser'");
        if($saveCuruser['cfgValue']!=$curuser&&$curuser!="_qnap_support"){
           return array('status' => 3005, 'result' => null);
        }

        $curlUrl = $this->apiUrl."/oauth2/me";
        $header =array(
            'X-Qid-Authorization: '.$token['access_token'],
        );

        $srvOutput = curlWrapper('GET', $curlUrl, null,false,$header);
        if (is_array($srvOutput)) {
            $srvOutput = json_encode($srvOutput);
            $srvOutput = json_decode($srvOutput);
        }else{
            $srvOutput = json_decode($srvOutput);
        }

        return $srvOutput;
        
    }

    public function getBindQid(){
        $result = $this->db->querySingleRow("SELECT cfgValue FROM configuration WHERE cfgKey='qidBindAccount'");
        return $result['cfgValue'];
    }
    public function getBindUserId(){
        $result = $this->db->querySingleRow("SELECT cfgValue FROM configuration WHERE cfgKey='qidBindUserID'");
        return $result['cfgValue'];
    }

    public function bind_user_info($curuser){
        $token = $this->getCachedToken('qidBind');
        if(!$token['access_token']){
            return array('status' => 3000, 'result' => null);
        }   
        if($this->isTokenExpires('qidBind')){
            return array('status' => 3004, 'result' => null);
        }
        $saveCuruser = $this->db->querySingleRow("SELECT cfgValue FROM configuration WHERE cfgKey='qidBindCuruser'");
        if($saveCuruser['cfgValue']!=$curuser&&$curuser!="_qnap_support"){
           return array('status' => 3005, 'result' => null);
        }

        $curlUrl = $this->apiUrl."/oauth2/me";
        $header =array(
            'X-Qid-Authorization: '.$token['access_token'],
        );

        $srvOutput = curlWrapper('GET', $curlUrl, null,false,$header);
        if (is_array($srvOutput)) {
            $srvOutput = json_encode($srvOutput);
            $srvOutput = json_decode($srvOutput);
        }else{
            $srvOutput = json_decode($srvOutput);
            if(isset($srvOutput->result->user_id)){
                $this->db->query("INSERT OR REPLACE INTO configuration ('cfgKey','cfgValue') VALUES ('qidBindUserID',?)",[$srvOutput->result->user_id]);
            }
        }

        return $srvOutput;
        
    }
    public function getCachedToken($key = 'qid'){
        $isExpired = $this->isTokenExpires($key);
        if($isExpired){
           return [ 'access_token'=>null, 'refresh_token'=>null,'expires_in'=>null ];
        }
        $accessToken = $this->db->querySingleRow("SELECT cfgValue FROM configuration WHERE cfgKey='{$key}AccessToken'");
        if($accessToken['cfgValue']==null){
            $this->db->querySingleRow("INSERT OR REPLACE INTO configuration ('cfgKey','cfgValue') VALUES ('{$key}AccessToken', 0)");
        }
        $refreshToken = $this->db->querySingleRow("SELECT cfgValue FROM configuration WHERE cfgKey='{$key}RefreshToken'");
        if($refreshToken['cfgValue']==null){
            $this->db->querySingleRow("INSERT OR REPLACE INTO configuration ('cfgKey','cfgValue') VALUES ('{$key}RefreshToken', 0)");
        }
        $expiresIn = $this->db->querySingleRow("SELECT cfgValue FROM configuration WHERE cfgKey='{$key}ExpiresIn'");
        if($expiresIn['cfgValue']==null){
            $this->db->querySingleRow("INSERT OR REPLACE INTO configuration ('cfgKey','cfgValue') VALUES ('{$key}ExpiresIn', 0)");
        }
        return [ 'access_token'=>$accessToken['cfgValue'], 'refresh_token'=>$refreshToken['cfgValue'],'expires_in'=>$expiresIn['cfgValue'] ];
    }
    public function isTokenExpires($key = 'qid'){
        $expires = $this->db->querySingleRow("SELECT cfgValue FROM configuration WHERE cfgKey='{$key}ExpiresIn'");

        if($expires['cfgValue']==null){
            $isExpires = true;
        }else{
            if( (time()-$expires['cfgValue']) > 60*1000 ) { 
                $refreshTokenResult = $this->refreshToken($key);
                if($refreshTokenResult === true){
                    $isExpires = false; 
                }else{
                    $isExpires = true;
                }
            }else{
                $isExpires = false;
            }
        }
        return $isExpires;
    }

    public function refreshToken($key = 'qid'){
        $token = $this->getCachedToken($key);
        if(!$token['refresh_token']){
            return false;
        }   
        if( $token['expires_in'] && ($token['expires_in'] - 60*60*2 >time())){
            return false;
        }else{
            $this->fileLogModel->logDebug($key.":access_token expired", __FILE__, __LINE__);
        }
        $curlUrl = $this->apiUrl . '/oauth2/refresh';
        $postData = array(
            "grant_type"=>"refresh_token",
            "refresh_token"=>$token['refresh_token'],
        );
        $curlPostFields = json_encode($postData);
        $srvOutput = curlWrapper('POST', $curlUrl, $curlPostFields, null, ['Content-Type: application/json']);
        if (is_array($srvOutput)) {
            $srvOutput = json_encode($srvOutput);
            $srvOutput = json_decode($srvOutput);
        }else{
            $srvOutput = json_decode($srvOutput);
        }

        if($srvOutput->access_token){
            $expires = time()+$srvOutput->expires_in;
            $this->db->query("INSERT OR REPLACE INTO configuration ('cfgKey','cfgValue') VALUES ('{$key}AccessToken',?)",[$srvOutput->access_token]);
            $this->db->query("INSERT OR REPLACE INTO configuration ('cfgKey','cfgValue') VALUES ('{$key}RefreshToken',?)",[$srvOutput->refresh_token]);
            $this->db->query("INSERT OR REPLACE INTO configuration ('cfgKey','cfgValue') VALUES ('{$key}ExpiresIn',?)",[$expires]);
            return true;
        }
        return  false;
    }
}
