<?php

namespace App\Models;

class RemoteAccessModel extends \LMVC\System\Model
{
    protected $authedKeyPath = '/home/_qnap_support/.ssh/authorized_keys';

    public $qtsUserID = '_qnap_support';

    public function __construct()
    {
        parent::__construct();

        $this->apiGWUrl = "https://".config('api_gateway_server') . '/help';
    }

    // {{{ public function getStatus(){
    public function getStatus()
    {
        $row = $this->db->querySingleRow('SELECT * FROM remoteAccessStatus ORDER BY startTime DESC LIMIT 1');

        return $row;
    }
    // }}}

    // {{{ public function getActivatedStatus(){
    public function getActivatedStatus()
    {
        $row = $this->db->querySingleRow('SELECT * FROM remoteAccessStatus WHERE status in (1001, 1002) ORDER BY cid DESC LIMIT 1');

        return $row;
    }
    // }}}

    // {{{ public function getTempAccount($ticketId = 0) {
    public function getTempAccount($ticketId = 0)
    {
        $dsID = $this->db->querySingleRow("SELECT cfgValue FROM configuration WHERE cfgKey='tempId'");
        $dsPW = $this->db->querySingleRow("SELECT cfgValue FROM configuration WHERE cfgKey='tempPw'");

        if (!empty($dsID)) {
            $dsID = $dsID['cfgValue'];
        } else {
            $dsID = null;
        }

        if (!empty($dsPW)) {
            $dsPW = $dsPW['cfgValue'];
        } else {
            $dsPW = null;
        }

        return array('tempId' => $dsID, 'tempPw' => $dsPW);
    }
    // }}}

    // {{{ public function getForwardPort($ticketId = 0) {
    public function getForwardPort($ticketId = 0)
    {
        $ds = $this->db->query("SELECT sshForwardingPort, qtsForwardingPort, httpsForwardingPort, forwardingServer FROM remoteAccessStatus WHERE ticketId='{$ticketId}' ORDER BY cid DESC LIMIT 1");

        return $ds;
    }
    // }}}

    // {{{ public function isEnabled($ticketId = 0, $email = null){
    public function isEnabled($ticketId = 0, $email = null)
    {
        $isEnabled = $this->db->queryDataIsExist("SELECT cid FROM remoteAccessStatus WHERE status in (1001, 1002) AND ticketId='{$ticketId}' AND email='{$email}'");

        return $isEnabled;
    }
    // }}}

    // {{{ public function isExist($ticketId = 0, $email = null) {
    public function isExist($ticketId = 0, $email = null)
    {
        $isExist = $this->db->queryDataIsExist("SELECT cid FROM remoteAccessStatus WHERE ticketId='{$ticketId}' AND email='{$email}'");

        return $isExist;
    }
    // }}}

    // {{{ public function isExtended($ticketId = 0, $email = null) {
    public function isExtended($ticketId = 0, $email = null)
    {
        $isExtended = $this->db->queryDataIsExist("SELECT cid FROM remoteAccessStatus WHERE ticketId='{$ticketId}' AND email='{$email}' AND extended=1");

        return $isExtended;
    }
    // }}}

    // {{{ public function setTempAccount() {
    public function setTempAccount()
    {
        $this->db->queryNoneResult("UPDATE configuration SET cfgValue='{$this->qtsUserID}' WHERE cfgKey='tempId'");
        $this->db->queryNoneResult("UPDATE configuration SET cfgValue='{$this->qtsUserPW}' WHERE cfgKey='tempPw'");
    }
    // }}}

    // {{{ public function setStatus($status = 1000, $ticketId = 0, $email = null) {
    public function setStatus($status = 1000, $ticketId = 0, $email = null)
    {
        return $this->db->queryNoneResult("UPDATE remoteAccessStatus SET status='{$status}' WHERE ticketId='{$ticketId}' AND email='{$email}'");
    }
    // }}}

    // {{{ public function confirmFinished($ticketId = 0, $email = null) {
    public function confirmFinished($ticketId = 0, $email = null)
    {
        $this->db->queryNoneResult("UPDATE remoteAccessStatus SET confirmedStatus=1 WHERE ticketId='{$ticketId}' AND email='{$email}'");
    }
    // }}}

    // {{{ public function enable($ticketId = 0, $email = null, $startTime = null, $expireTime = null, $forwardIP = null, $forwardSSHPort = null, $forwardHttpPort = null){
    public function enable($ticketId = 0, $email = null, $startTime = null, $expireTime = null, $forwardIP = null, $forwardSSHPort = null, $forwardHttpPort = null, $forwardHttpsPort = null)
    {
        if ($this->isExist($ticketId, $email)) {
            $rowCount = $this->db->queryNoneResult("UPDATE remoteAccessStatus SET status='1001', startTime='{$startTime}', expireTime='{$expireTime}', forwardingServer='{$forwardIP}', sshForwardingPort='{$forwardSSHPort}', qtsForwardingPort='{$forwardHttpPort}', httpsForwardingPort='{$forwardHttpsPort}', extended=0, confirmedStatus=0 WHERE ticketId='{$ticketId}' AND email='{$email}'");
        } else {
            $rowCount = $this->db->queryNoneResult("INSERT INTO remoteAccessStatus (status, ticketId, email, startTime, expireTime, forwardingServer, sshForwardingPort, qtsForwardingPort, httpsForwardingPort, extended, confirmedStatus) VALUES ('1001', '{$ticketId}', '{$email}', '{$startTime}', '{$expireTime}', '{$forwardIP}', '{$forwardSSHPort}', '{$forwardHttpPort}', {$forwardHttpsPort}, 0, 0)");
        }
        if ($rowCount <= 0) {
            return false;
        }

        return true;
    }
    // }}}

    // {{{ public function setWaitActivate() {
    public function setWaitActivate()
    {
        return $this->db->queryNoneResult("UPDATE remoteAccessStatus SET status='1000' WHERE ticketId='{$this->ticketId}' AND email='{$this->email}'");
    }
    // }}}

    // {{{ public function setDisable() {
    public function setDisable()
    {
        return $this->db->queryNoneResult("UPDATE remoteAccessStatus SET status='1003' WHERE ticketId='{$this->ticketId}' AND email='{$this->email}'");
    }
    // }}}

    // {{{ public function setExpired() {
    public function setExpired()
    {
        return $this->db->queryNoneResult("UPDATE remoteAccessStatus SET status='1004' WHERE ticketId='{$this->ticketId}' AND email='{$this->email}'");
    }
    // }}}

    // {{{ public function setExpireTime($ticketId = 0, $email = null, $expireTime = null) {
    public function setExpireTime($ticketId = 0, $email = null, $expireTime = null)
    {
        $result = $this->db->queryNoneResult("UPDATE remoteAccessStatus SET expireTime='{$expireTime}', extended=1 WHERE ticketId='{$ticketId}' AND email='{$email}'");
    }
    // }}}

    // {{{ public function getExpireTime($ticketId = 0, $email = null) {
    public function getExpireTime($ticketId = 0, $email = null)
    {
        $result = $this->db->querySingleRow("SELECT expireTime FROM remoteAccessStatus WHERE ticketId='{$ticketId}' AND email='{$email}' ORDER BY cid DESC LIMIT 1");

        return $result;
    }
    // }}}

    //{{{ public function setClosedInfo(){
    public function setClosedInfo()
    {
        $remoteInfo = $this->getRemoteSessionInfo();

        if ($remoteInfo == false) {
            return false;
        }

        $closedTime = $remoteInfo->data->finished_time;
        $closedMsg = $remoteInfo->data->finished_report;

        $result = $this->db->queryNoneResult("UPDATE remoteAccessStatus SET status=1005, closedTime='{$closedTime}', closedMsg='{$closedMsg}' WHERE ticketId='{$this->ticketId}' AND email='{$this->email}'");

        return array('closedTime' => $closedTime, 'closedMsg' => $closedMsg);
    }
    //}}}

    // {{{ public function resetAuthedKey() {
    public function resetAuthedKey()
    {
        if (is_link($this->authedKeyPath)) {
            if (readlink($this->authedKeyPath) == '/dev/null') {
                unlink($this->authedKeyPath);

                return true;
            }
        }

        return false;
    }
    // }}}

    // {{{ public function restoreAuthedKey() {
    public function restoreAuthedKey()
    {
        if (file_exists($this->authedKeyPath)) {
            $replacedContent = '';

            $authKeyContent = file_get_contents($this->authedKeyPath);
            $pattern = '/.*' . $this->ticketId . '.*[\\r\\n]*/';
            $replacedContent = preg_replace($pattern, '', $authKeyContent);
            file_put_contents($this->authedKeyPath, $replacedContent);

            return true;
        }

        return false;
    }
    // }}}

    // {{{ public function checkQTSUserByAPI()
    public function checkQTSUserByAPI()
    {
        $isUserExist = true;
        $curlUrl = httpsOrNot() . "://localhost:{$this->qtsApiPort}/cgi-bin/wizReq.cgi?wiz_func=user_create&action=check_user&uname={$this->qtsUserID}";
        $curlPostFields = "sid={$this->sID}";

        $srvOutput = curlWrapper('POST', $curlUrl, $curlPostFields, true);

        if (!is_array($srvOutput)) {
            $xml = new \SimpleXMLElement($srvOutput, LIBXML_NOERROR | LIBXML_NOCDATA);
            $xmlUserExist = $xml->xpath('/QDocRoot/func/ownContent/return');
            $isUserExist = json_decode($xmlUserExist[0]);

            if ($isUserExist == 0) {
                $isUserExist = false;
            }

            return $isUserExist;
        }

        return false;
    }
    //}}}

    //{{{ public function checkQTSUser()
    public function checkQTSUser()
    {
        $isUserExist = true;

        $passwd = file_get_contents('/etc/passwd');

        if (strpos($passwd, $this->qtsUserID) === false) {
            $isUserExist = false;
        }

        return $isUserExist;
    }
    //}}}

    //{{{public function removeQTSUserByAPI()
    public function removeQTSUserByAPI()
    {
        // Remove QTS account
        $cmd="qsh privilege.'GET /privilege/user' '{\"__HEADER__\":{\"QUERY_STRING\":\"subfunc=user&apply=1&del_cnt=1&delete_userhome=true&user_list={$this->qtsUserID}\"}}'";
        $return_var = shell_exec($cmd);
        return $return_var;
    }
    //}}}

    //{{{ public function removeQTSUser()
    public function removeQTSUser()
    {
        shell_exec("/usr/sbin/userdel {$this->qtsUserID}");
    }
    //}}}

    //{{{ public function createQTSUserByAPI() {
    public function createQTSUserByAPI()
    {
        $base64PW = base64_encode($this->qtsUserPW);

        $cmd="qsh privilege.'GET /privilege/user' '{\"__HEADER__\":{\"QUERY_STRING\":\"wiz_func=user_create&action=add_user&set_application_privilege=1&rd_share_len=1&rd_share0=Public&rw_share_len=0&no_share_len=0&a_tel=&email=&ps=&gp_len=2&gp0=administrators&gp1=everyone&hidden=no&oplocks=1&create_priv=0&comment=&vol_no=2&a_email=&a_description=&recursive=1&send_mail=0&AFP=1&FTP=1&SAMBA=1&WEBDAV=1&QBOX=1&WFM=1&recycle_bin=0&recycle_bin_administrators_only=0&a_username={$this->qtsUserID}&a_passwd={$base64PW}\"}}'";
        $return_var = shell_exec($cmd);
        return $return_var;
    }
    //}}}

    //{{{ public function createQTSUser() {
    public function createQTSUser()
    {
        shell_exec("/usr/sbin/userdel/useradd -g administrators -s /bin/bash -p $(echo {$this->qtsUserPW} | openssl passwd -1 -stdin) {$this->qtsUserID}");
    }
    //}}}

    public function setQTSUserConfig(){
        $filename = "/etc/config/.qos_config/users/_qnap_support/config";
        $dirname = dirname($filename);
        if (!is_dir($dirname))
        {
            mkdir($dirname, 0755, true);
        }
        $file = fopen($filename ,"w");
        fwrite($file,"
[common]
read_privacy_version = 1.0
show_tour = false
autoShowStorageV2 = false
I_know_raid_scrubbing = true
auto_open_help = false
autoShowLicensePromote = false
I_know_has_not_support_snapshot_space = true

[appCenterQuick]
status = true

[appCenter]
iknow = true
");
        fclose($file);

    }

    public function getRemoteSessionEnable($email,$supportID,$supportPW,$httpPort,$httpsPort,$sshPort,$base64SK){
        $curlUrl = $this->apiGWUrl . "/apps/qdesk_service/api/v1/qpkg/remote/{$this->ticketId}/enable";
        $postData = array(
            "firmware_type"=>"QNE",
            "email"=>$email,
            "login_id"=>$supportID,
            "password"=>$supportPW,
            "http_port"=>$httpPort,
            "https_port"=>$httpsPort,
            "ssh_port"=>$sshPort,
            "private_key"=>$base64SK
        );
        $curlPostFields = http_build_query($postData);

        $token = $this->qcloudModel->getCachedToken();
        if(!$token['access_token']){
            return array('status' => 3000, 'result' => null);
        }
        $header =array(
            'X-Qid-Authorization: '.$token['access_token'],
            'Content-Type: application/x-www-form-urlencoded'
        );
        $srvOutput = curlWrapper('POST', $curlUrl, $curlPostFields,false,$header);
        return $srvOutput;
    }

    //{{{ public function getRemoteSessionStatus() {
    public function getRemoteSessionStatus()
    {
        $curlUrl = $this->apiGWUrl . "/apps/qdesk_service/api/v1/qpkg/remote/{$this->ticketId}";
        $token = $this->qcloudModel->getCachedToken();
        if(!$token['access_token']){
            return array('status' => 403, 'result' => null, 'msg' => 'no access_token');
        }
        $header =array(
            'X-Qid-Authorization: '.$token['access_token'],
        );
        $srvOutput = curlWrapper('GET', $curlUrl, null, false, $header);
        if (is_array($srvOutput)) {
            $srvOutput = json_encode($srvOutput);
        }
        $srvOutput = json_decode($srvOutput);

        if (isset($srvOutput->status) && $srvOutput->status === 'FALSE') {
            if (isset($srvOutput->result->errCode) && $srvOutput->result->errCode == 28) {
                return array('status' => '408', 'msg' => $srvOutput->result->errMsg);
            } else {
                return $srvOutput->status;
            }
        }

        $ticketStatus = $srvOutput->status;
        if ($ticketStatus == 0) {
            $ticketStatus = $srvOutput->data->status;
        }

        return $ticketStatus;
    }
    //}}}

    //{{{ public function getRemoteSessionInfo() {
    public function getRemoteSessionInfo()
    {
        $curlUrl = $this->apiGWUrl . "/apps/qdesk_service/api/v1/qpkg/remote/{$this->ticketId}";
        $token = $this->qcloudModel->getCachedToken();
        if(!$token['access_token']){
            return array('status' => 403, 'result' => null);
        }
        $header =array(
            'X-Qid-Authorization: '.$token['access_token'],
        );
        $srvOutput = curlWrapper('GET', $curlUrl, null, false, $header);
        if (is_array($srvOutput)) {
            $srvOutput = json_encode($srvOutput);
        }
        $srvOutput = json_decode($srvOutput);

        if (isset($srvOutput->status) && $srvOutput->status === 'FALSE') {
            if (isset($srvOutput->result->errCode) && $srvOutput->result->errCode == 28) {
                return array('status' => '408', 'msg' => $srvOutput->result->errMsg);
            } else {
                $this->fileLogModel->logError('Can not get remote session info. HTTP StatusCode: ' . $srvOutput->httpStatusCode, __FILE__, __LINE__);

                return false;
            }
        }

        return $srvOutput;
    }
    //}}}


    //{{{ public function generateSSHKeypair() {
    public function generateSSHKeypair()
    {
        // Remote SSH keypair for this ticket
        exec("qsh helpdeskd.genKey -v '{\"path\":\"".$this->tmpKeyPath."\"}'");
        $publicKey = shell_exec("/usr/bin/ssh-keygen -y -f $this->tmpKeyPath");
        $privateKey = file_get_contents($this->tmpKeyPath);
        $base64SK = base64_encode($privateKey);

        return array('encSK' => $base64SK, 'PK' => $publicKey);
    }
    //}}}

    //{{{ public function renewSSHKeypair() {
    public function renewSSHKeypair()
    {
        // Update authorized_keys
        $pattern = '/[\\r\\n]/';
        $pkWithTicketID = preg_replace($pattern, '', $this->publicKey) . ' ' . $this->ticketId . "\r\n";

        // Send private key to Helpdesk server
        $serverUrl = $this->apiGWUrl .  "/apps/qdesk_service/api/v1/qpkg/remote/{$this->ticketId}/renewkey";
        $dataPostFields = "privateKey={$this->base64SK}";
        $token = $this->qcloudModel->getCachedToken();
        if(!$token['access_token']){
            return array('status' => 3000, 'result' => null);
        }
        $header =array(
            'X-Qid-Authorization: '.$token['access_token'],
        );
        $srvOutput = curlWrapper('POST', $serverUrl, $dataPostFields, false, $header);

        if (is_array($srvOutput)) {
            $srvOutput = json_encode($srvOutput);
        }
        $srvOutput = json_decode($srvOutput);

        if (isset($srvOutput->status) && $srvOutput->status === 'FALSE') {
            if (isset($srvOutput->result->errMsg)) {
                return array('status' => 'FALSE', 'result' => $srvOutput->result->errMsg);
            } else {
                return array('status' => 'FALSE', 'result' => $srvOutput->result);
            }
        } else {
            file_put_contents($this->authedKeyPath, $pkWithTicketID, FILE_APPEND);
            return true;
        }
    }
    //}}}

    //{{{ public function generateFwKeypair() {
    public function generateFwKeypair()
    {
        // Remove exist forwarding keypair
        if (file_exists($this->fwSKeyPath)) {
            unlink($this->fwSKeyPath);
        }

        // Generate forwarding keypair
        shell_exec("/usr/bin/ssh-keygen -b 2048 -t rsa -f $this->fwSKeyPath -q -N ''");
        shell_exec("/bin/chmod 600 $this->fwSKeyPath");
        $fwPKey = shell_exec("/usr/bin/ssh-keygen -y -f $this->fwSKeyPath");
        $fwPKey = str_replace("\n", ' ' . $this->ticketId . "\n", $fwPKey);
        $base64FwPK = base64_encode($fwPKey);

        // Send keypair to forwarding server
        $fwDaemonUrl = $this->apiGWUrl . "/api/v1/remote/{$this->ticketId}/enable";
        $postData = array(
            'fwsrv_publickey'=>$base64FwPK,
            'email'=>$this->email,
        );
        $curlPostFields = http_build_query($postData);
        $token = $this->qcloudModel->getCachedToken();
        if(!$token['access_token']){
            return array('status' => 3000, 'result' => null);
        }
        $header =array(
            'X-Qid-Authorization: '.$token['access_token'],
        );
        $srvOutput = curlWrapper('POST', $fwDaemonUrl, $curlPostFields,false,$header);

        return $srvOutput;
    }
    //}}}

    public function getExtendExpired(){
        $curlUrl = $this->apiGWUrl . "/apps/qdesk_service/api/v1/qpkg/remote/{$this->ticketId}/delayexpired";
        $token = $this->qcloudModel->getCachedToken();
        if(!$token['access_token']){
            return array('status' => 3000, 'result' => null);
        }
        $header =array(
            'X-Qid-Authorization: '.$token['access_token'],
        );
        $srvOutput = curlWrapper('GET', $curlUrl, null, false, $header);
        if (is_array($srvOutput)) {
            $srvOutput = json_encode($srvOutput);
        }
        $srvOutput = json_decode($srvOutput);

        return $srvOutput;
    }

    //{{{ public function establishSSHFwTunnel() {
    public function establishSSHFwTunnel()
    {
        if (isset($this->sshFwPort) && isset($this->sshPort) && isset($this->forwardIP) && isset($this->forwardSrvPort) && isset($this->fwSKeyPath)) {
           // Fix Bug 93533 - [Helpdesk_Service] SSH tunnel keep /share/MD0_DATA/homes/admin in use and unable to umount data volume
            $helpdesk_PATH = str_replace("\n", '', shell_exec('/sbin/getcfg -f /etc/config/qpkg.conf helpdesk Install_Path'));
            $helpdesk_PATH="/usr/share/qdesk";
            $cmdSSHFw = "cd $helpdesk_PATH; /usr/bin/ssh -NfR {$this->sshFwPort}:localhost:{$this->sshPort} ubuntu@{$this->forwardIP} -p {$this->forwardSrvPort} -i {$this->fwSKeyPath} -o ConnectTimeout=60 -o ServerAliveInterval=60 -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no -o LogLevel=quiet -o ExitOnForwardFailure=yes > /dev/null 2>/dev/null &";
            shell_exec($cmdSSHFw);

            return true;
        }

        return false;
    }
    //}}}

    //{{{ public function establishHTTPFwTunnel() {
    public function establishHTTPFwTunnel()
    {
        if (isset($this->httpFwPort) && isset($this->sshPort) && isset($this->forwardIP) && isset($this->forwardSrvPort) && isset($this->fwSKeyPath)) {
            // Fix Bug 93533 - [Helpdesk_Service] SSH tunnel keep /share/MD0_DATA/homes/admin in use and unable to umount data volume
            $helpdesk_PATH = str_replace("\n", '', shell_exec('/sbin/getcfg -f /etc/config/qpkg.conf helpdesk Install_Path'));
            $helpdesk_PATH="/usr/share/qdesk";
            $cmdHTTPFw = "cd $helpdesk_PATH; /usr/bin/ssh -NfR {$this->httpFwPort}:localhost:{$this->httpPort} ubuntu@{$this->forwardIP} -p {$this->forwardSrvPort} -i {$this->fwSKeyPath} -o ConnectTimeout=60 -o ServerAliveInterval=60 -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no -o LogLevel=quiet -o ExitOnForwardFailure=yes > /dev/null 2>/dev/null &";
            shell_exec($cmdHTTPFw);

            return true;
        }

        return false;
    }
    //}}}

    //{{{ public function establishHTTPSFwTunnel() {
    public function establishHTTPSFwTunnel()
    {
        if (isset($this->httpsFwPort) && isset($this->sshPort) && isset($this->forwardIP) && isset($this->forwardSrvPort) && isset($this->fwSKeyPath)) {
            // Fix Bug 93533 - [Helpdesk_Service] SSH tunnel keep /share/MD0_DATA/homes/admin in use and unable to umount data volume
             $helpdesk_PATH = str_replace("\n", '', shell_exec('/sbin/getcfg -f /etc/config/qpkg.conf helpdesk Install_Path'));
            $helpdesk_PATH="/usr/share/qdesk";
            $cmdHTTPSFw = "cd $helpdesk_PATH; /usr/bin/ssh -NfR {$this->httpsFwPort}:localhost:{$this->httpsPort} ubuntu@{$this->forwardIP} -p {$this->forwardSrvPort} -i {$this->fwSKeyPath} -o ConnectTimeout=60 -o ServerAliveInterval=60 -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no -o LogLevel=quiet -o ExitOnForwardFailure=yes > /dev/null 2>/dev/null &";
            shell_exec($cmdHTTPSFw);

            return true;
        }

        return false;
    }
    //}}}

    //{{{ public function getSSHTunnelPid() {
    public function getSSHTunnelPid()
    {
        if (empty($this->sshFwPort) || empty($this->httpFwPort) || empty($this->httpsFwPort)) {
            return 'FALSE';
        }

        $sshFwPid = shell_exec("/bin/ps ax | grep 'NfR {$this->sshFwPort}:localhost' | grep 'ubuntu@' | grep -v 'grep' | awk '{print $1}'");
        $httpFwPid = shell_exec("/bin/ps ax | grep 'NfR {$this->httpFwPort}:localhost' | grep 'ubuntu@' | grep -v 'grep' | awk '{print $1}'");
        $httpsFwPid = shell_exec("/bin/ps ax | grep 'NfR {$this->httpsFwPort}:localhost' | grep 'ubuntu@' | grep -v 'grep' | awk '{print $1}'");
        $sshFwPid = str_replace("\n", '', $sshFwPid);
        $httpFwPid = str_replace("\n", '', $httpFwPid);
        $httpsFwPid = str_replace("\n", '', $httpsFwPid);

        return array('sshFwPid' => $sshFwPid, 'httpFwPid' => $httpFwPid, 'httpsFwPid' => $httpsFwPid);
    }
    //}}}

    // {{{ public function killSSHTunnel() {
    public function killSSHTunnel()
    {
        $fwPid = $this->getSSHTunnelPid();

        if ($fwPid !== false && (!empty($fwPid['sshFwPid']) || !empty($fwPid['httpFwPid']) || !empty($fwPid['httpsFwPid']))) {
            shell_exec("/bin/kill -9 {$fwPid['sshFwPid']} {$fwPid['httpFwPid']} {$fwPid['httpsFwPid']}");

            return true;
        }

        return false;
    }
    // }}}

    // {{{ public function removeFwSSHPKey() {
    public function removeFwSSHPKey()
    {
        $curlUrl = $this->apiGWUrl . "/api/v1/remote/{$this->ticketId}/disable";
        $curlPostFields = "ticketId={$this->ticketId}";
        $token = $this->qcloudModel->getCachedToken();
        if(!$token['access_token']){
            return array('status' => 3000, 'result' => null);
        }
        $header =array(
            'X-Qid-Authorization: '.$token['access_token'],
        );
        curlWrapper('POST', $curlUrl, $curlPostFields, false, $header);
    }
    // }}}

    // {{{ public function setWorkScreen($isOn) {
    public function setWorkScreen($isOn)
    {
        $profilePath = '/root/.profile';
        $pattern = "/\nexport TERM=xterm\nscreen -Rd helpdesk\n/i";

        if ($isOn) {
            if (file_exists($profilePath)) {
                $profile = file_get_contents($profilePath);
                if (!preg_match($pattern, $profile)) {
                    $profile = $profile . "\nexport TERM=xterm\nscreen -Rd helpdesk\n";
                    file_put_contents($profilePath, $profile);
                }
            }
        } else {
            if (file_exists($profilePath)) {
                $profile = file_get_contents($profilePath);
                $profile = preg_replace($pattern, '', $profile);
                file_put_contents($profilePath, $profile);
            }
        }
    }
    // }}}

    // {{{ public function setRemoteSessionDisable() {
    public function setRemoteSessionDisable()
    {
        $curlUrl = $this->apiGWUrl . "/apps/qdesk_service/api/v1/qpkg/remote/{$this->ticketId}/disable";
        $curlPostFields = "ticketId={$this->ticketId}";
        $token = $this->qcloudModel->getCachedToken();
        if(!$token['access_token']){
            return array('status' => 3000, 'result' => null);
        }
        $header =array(
            'X-Qid-Authorization: '.$token['access_token'],
        );
        $srvOutput = curlWrapper('POST', $curlUrl, $curlPostFields, false, $header);
        if (is_array($srvOutput)) {
            $srvOutput = json_encode($srvOutput);
        }
        $srvOutput = json_decode($srvOutput);

        if (isset($srvOutput->status) && $srvOutput->status === 'FALSE') {
            if (isset($srvOutput->result->errCode) && $srvOutput->result->errCode == 28) {
                return array('status' => '408', 'msg' => $srvOutput->result->errMsg);
            } else {
                return array('status' => 'FALSE', 'result' => $srvOutput->result);
            }
        }

        return true;
    }
    // }}}

    // {{{ public function disableRemoteSession() {
    public function disableRemoteSession()
    {
        // Kill Reverse SSH tunnel
        $this->killSSHTunnel();

        // Remove SSH publickey from forwarding server
        $this->removeFwSSHPKey();

        // Restore .ssh/authorized_keys
        $this->resetAuthedKey();
        $this->restoreAuthedKey();

        // Remove support account -- Fix #75074 @Bugzilla
        if ($this->checkQTSUser()) {
            $this->removeQTSUser();
        }

        // Close work screen
        $this->setWorkScreen(0);
    }
    // }}}

    public function getReportResultLog()
    {
        $result = $this->db->querySingleRow("SELECT cfgValue FROM configuration WHERE cfgKey='showRemoteResultLog'");
        return $result['cfgValue'];
    }
    public function enableReportResultLog()
    {
        $this->db->queryNoneResult("INSERT OR REPLACE INTO configuration ('cfgKey','cfgValue') VALUES ('showRemoteResultLog', 1)");
    }
    public function disableReportResultLog()
    {
        $this->db->queryNoneResult("INSERT OR REPLACE INTO configuration ('cfgKey','cfgValue') VALUES ('showRemoteResultLog', 0)");
    }
}
