<?php
namespace LMVC\System;

use LMVC\System\Foundation\Singleton;
use LMVC\System\Config;
use LMVC\System\Router;
use LMVC\System\Toolbox;
use LMVC\System\Lang;
use LMVC\System\Response;



class App
{

	public static $controllerClassName = null;
	public static $controllerClassMethod = null;
	public static $controllerClassMethodArg = null;
	public static $lang = null;
	public static $config = null;
	public static $router = null;
	public static $status = null;




	public function __construct($configPath, $routerPath) {

		set_error_handler(array($this, 'errorHandler'));

		static::$router = Router::getInstance();
		require_once($routerPath);

		static::$config = Config::getInstance();
		require_once($configPath);



		static::$lang = new Lang();
	}

	public function errorHandler($errno, $errstr=null, $errfile=null, $errline=null, $errcontext=null, $errorTypes=null){
		if (!(error_reporting() & $errno)) {
			// This error code is not included in error_reporting
			return;
		}

		if(empty($response_code)){
			$response_code = 500;
		}


		Response::http_response_code($response_code);

        @$errcontext = (string)$errcontext;

		$now = date("Y-m-d H:i:s");
		$errLog = 	"errorCode: $errno\t".
					"errMessage: $errstr\t".
					"errorFile: $errfile\t".
					"errorLine: $errline\t".
					"errorContext: $errcontext\t".
					"errorTypes: $errorTypes\t";
		Toolbox::log('error', $errLog);

		header('Content-Type: application/json; charset=utf-8');
		echo json_encode(
			array(
				'errorCode' => $errno,
				'message' => $errstr,
				'errorFile' => $errfile,
				'errorLine' => $errline,
				'errorContext' => $errcontext,
				'errorTypes' => $errorTypes
			)
		);
		exit;
	}


	public function request(){
		if (PHP_SAPI != "cli") {
			//非cli mode
		    $method = $_SERVER['REQUEST_METHOD'];
			$uri = $_SERVER['REQUEST_URI'];
			$uri = substr($uri, strlen(static::$config->getItem('base_uri')));

			$get_para_pos = strpos($uri, '?');
			if($get_para_pos!==false){
				$uri = substr($uri, 0, $get_para_pos);
			}
		}else{
			//cli mode
			$method = 'CLI';
			$argv = $GLOBALS["argv"];
			$uri='';
			$arg_num = count($argv);
			if($arg_num>1){
				for($i=1; $i<$arg_num; $i++){
					$uri .= '/'.$argv[$i];
				}
			}

			$time_zone = $this->get_sys_timezone();
			date_default_timezone_set($time_zone);
		}


		list(	static::$controllerClassName,
				static::$controllerClassMethod,
				static::$controllerClassMethodArg) = static::$router->route($method, $uri);
	}

	public function response(){
		$controller = new static::$controllerClassName();
		if(static::$controllerClassMethodArg===null){
			call_user_func_array(array($controller, static::$controllerClassMethod), array());
		}else{
			call_user_func_array(array($controller, static::$controllerClassMethod), static::$controllerClassMethodArg);
		}

	}

	public function error($message, $response_code=500){

		if($response_code!=200){
			Toolbox::log('error', "{$response_code} {$message}");
		}



		Response::http_response_code($response_code);
		echo $response_code.' - '.$message;
		exit;
	}

	private function get_sys_timezone()
	{
		$result = 'Asia/Taipei';

		$zone = array(
				'MHT12'=>'Pacific/Kwajalein',
				'SST11'=>'Pacific/Samoa',
				'HST10'=>'US/Hawaii',
				'AKST9AKDT8'=>'America/Anchorage',
				'PST8PDT7'=>'America/Los_Angeles',
				'MST7'=>'America/Phoenix',
				'MST7MDT6'=>'America/Phoenix',
				'CST6'=>'America/Chicago',
				'CST6CDT5'=>'America/Mexico_City',
				'COT5'=>'America/Lima',
				'EST5EDT4'=>'America/New_York',
				'EST5'=>'America/Indiana/Vevay',
				'AST4ADT3'=>'Canada/Atlantic',
				'VET4'=>'America/Caracas',
				'CLST3CLT4,M3.2.0/00:00:00,M10.2.0/00:00:00'=>'America/Santiago',
				'NST3:30NDT2:30'=>'Canada/Newfoundland',
				'BRST2BRT3,M2.3.0/00:00:00,M10.2.0/00:00:00'=>'America/Sao_Paulo',
				'ARST3ART4,M3.1.0/00:00:00,M10.1.0/00:00:00'=>'America/Argentina/Buenos_Aires',
				'GST2'=>'Etc/GMT-2',
				'AZOT1AZOST0'=>'Atlantic/Azores',
				'WET0'=>'Africa/Casablanca',
				'GMT0IST-1','Europe/Dublin',
				'CET-1CEST-2','Europe/Rome',
				'EET-2EEST-3'=>'Europe/Athens',
				'EET-2'=>'Europe/Bucharest',
				'EET-2EEST-3,M4.5.5/00:00:00,M9.5.4/23:00:00'=>'Africa/Cairo',
				'CAT-2'=>'Africa/Harare',
				'EET-2EEST-3'=>'Europe/Sofia',
				'IST-2IDT-3,M3.5.5/01:00:00,273/01:00:00'=>'Asia/Jerusalem',
				'ADT-3'=>'Asia/Kuwait',
				'EAT-3'=>'Africa/Nairobi',
				'FET-3'=>'Europe/Minsk',
				'GST-4'=>'Asia/Muscat',
				'GET-4GEST-5'=>'Asia/Baku',
				'MSK-4'=>'Europe/Moscow',
				'AFT-4:30'=>'Asia/Kabul',
				'PKT-5'=>'Asia/Karachi',
				'IST-5:30'=>'Asia/Dili',
				'LKT-5:30'=>'Asia/Colombo',
				'ALMT-6ALMST-7'=>'Asia/Almaty',
				'YEKT-6'=>'Asia/Yekaterinburg',
				'MMT'=>'Asia/Rangoon',
				'ICT-7'=>'Asia/Bangkok',
				'HKT-8'=>'Asia/Hong_Kong',
				'WST-8,M10.5.0/2,M3.5.0/3'=>'Australia/Perth',
				'SGT-8'=>'Asia/Singapore',
				'CST-8'=>'Asia/Taipei',
				'JST-9'=>'Asia/Tokyo',
				'KST-9'=>'Asia/Seoul',
				'CST-9:30'=>'Australia/Adelaide',
				'CST-9:30CDT,M10.1.0/2,M4.1.0/3'=>'Australia/Darwin',
				'EST-10'=>'Australia/Brisbane',
				'EST-10EDT,M10.1.0/2,M4.1.0/3'=>'Australia/Sydney',
				'GST-10'=>'Pacific/Guam',
				'EST-10EDT,M10.1.0/2,M4.1.0/3'=>'Australia/Hobart',
				'YAKT-10'=>'Asia/Yakutsk',
				'VLAT-11'=>'Asia/Vladivostok',
				'NZDT-13NZST-12,M3.3.0/02:00:00,M10.1.0/02:00:00'=>'Pacific/Auckland',
				'FJST-13FJT-12,M2.5.0/02:00:00,M11.1.0/02:00:00'=>'Pacific/Fiji',
				'MAGT-12'=>'Asia/Magadan'
			);
		/*
		$tz = file_get_contents('/etc/TZ');
		$tz = trim($tz);
		if(isset($zone[$tz]))
		{
			$result = $zone[$tz];
		}
		*/
		$result='Asia/Taipei';
		return $result;
	}



}
